@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: qt/checklists.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/checklists.cpp   3.3.6   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -46,10 +46,10 @@ CheckLists::CheckLists( QWidget *parent, const char *name )
     // create a list with 4 ListViewItems which will be parent items of other ListViewItems
     QValueList<QListViewItem *> parentList;
 
-    parentList.append( new QListViewItem( lv1, "Parent Item 1" ) );
-    parentList.append( new QListViewItem( lv1, "Parent Item 2" ) );
-    parentList.append( new QListViewItem( lv1, "Parent Item 3" ) );
-    parentList.append( new QListViewItem( lv1, "Parent Item 4" ) );
+    parentList.append( new QCheckListItem( lv1, "Parent Item 1", QCheckListItem::CheckBoxController ) );
+    parentList.append( new QCheckListItem( lv1, "Parent Item 2", QCheckListItem::CheckBoxController ) );
+    parentList.append( new QCheckListItem( lv1, "Parent Item 3", QCheckListItem::CheckBoxController ) );
+    parentList.append( new QCheckListItem( lv1, "Parent Item 4", QCheckListItem::CheckBoxController ) );
 
     QListViewItem *item = 0;
     unsigned int num = 1;
