@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/main.cpp   3.0.5   edited May 7 17:30 $
+** $Id: qt/main.cpp   3.3.6   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -26,7 +26,7 @@ const int numCols = 10;
 
 int main( int argc, char **argv )
 {
-    QApplication app( argc, argv );			
+    QApplication app( argc, argv );
 
     QTable table( numRows, numCols );
 
@@ -34,7 +34,7 @@ int main( int argc, char **argv )
     header->setLabel( 0, QObject::tr( "Tiny" ), 40 );
     header->setLabel( 1, QObject::tr( "Checkboxes" ) );
     header->setLabel( 5, QObject::tr( "Combos" ) );
-    header->setMovingEnabled(TRUE);
+    table.setColumnMovingEnabled(TRUE);
 
     QImage img( qtlogo_xpm );
     QPixmap pix = img.scaleHeight( table.rowHeight(3) );
@@ -48,7 +48,7 @@ int main( int argc, char **argv )
 	QComboTableItem * item = new QComboTableItem( &table, comboEntries, FALSE );
 	item->setCurrentItem( i % 4 );
 	table.setItem( i, 5, item );
-    }	
+    }
     for ( int j = 0; j < numRows; ++j )
 	table.setItem( j, 1, new QCheckTableItem( &table, "Check me" ) );
 
