@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/qdragapp.cpp   3.0.5   edited Mar 19 09:28 $
+** $Id: qt/qdragapp.cpp   3.3.6   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -12,6 +12,7 @@
 #include "qptrlist.h"
 #include "qintdict.h"
 #include "qpopupmenu.h"
+#include "qguardedptr.h"
 #include "qcolor.h"
 #include "qwidget.h"
 #include "qfontmetrics.h"
@@ -81,7 +82,7 @@ private:
     bool		   killingDrop;
     bool		   sendingChild;
     QWidget		  *clickedWidget;
-    QWidget		  *hostWidget;
+    QGuardedPtr<QWidget>   hostWidget;
     QCursor		   cursor;
 
     QPopupMenu*		   menu;
@@ -108,7 +109,7 @@ QDragApplication::~QDragApplication()
 bool QDragApplication::notify( QObject *o, QEvent *e )
 {
     if ( dragger && !dragger->notify( o, e ) )
-	return o->event( e );
+	return QApplication::notify( o, e );
     else
 	return FALSE;
 }
@@ -131,7 +132,6 @@ void DropWindow::closeEvent( QCloseEvent *e )
 QDragger::QDragger()
 {
     dragInfo.w	 = 0;
-    hostWidget	 = 0;
     killingDrop	 = FALSE;
     sendingChild = FALSE;
     draggedDict.setAutoDelete( TRUE );
@@ -384,7 +384,8 @@ void QDragger::grabFinished()
 {
     killingDrop	 = FALSE;
     sendingChild = FALSE;
-    hostWidget->setCursor( cursor );
+    if(hostWidget)
+	hostWidget->setCursor( cursor );
 }
 
 void QDragger::closeDropWindow( DropWindow *w )
