@@ -1,3 +1,3 @@
-#error "/tmp/qt-3.0-reggie-28534/qt/extensions/nsplugin/src/jri.h must be provided by Netscape"
+#error "/home/espenr/tmp/qt-3.3.6-espenr-14571/qt/extensions/nsplugin/src/jri.h must be provided by Netscape"
 #error "This file is just a placeholder. Please see the documentation"
 #error "to learn how to obtain the real file"
