@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/msvc_objectmodel.cpp   3.3.5   edited Aug 31 12:13 $
+** $Id: qt/msvc_objectmodel.cpp   3.3.6   edited Feb 7 15:47 $
 **
 ** Implementation of VCProject class.
 **
@@ -165,6 +165,7 @@ const char* _Name4				= "\n\t\t\t\tName=\"";
 const char* _Name5				= "\n\t\t\t\t\tName=\"";
 const char* _ObjectFile 			= "\n\t\t\t\tObjectFile=\"";
 const char* _OmitFramePointers 			= "\n\t\t\t\tOmitFramePointers=\"";
+const char* _OpenMP                             = "\n\t\t\t\tOpenMP=\"";
 const char* _Optimization 			= "\n\t\t\t\tOptimization =\"";
 const char* _OptimizeForProcessor 		= "\n\t\t\t\tOptimizeForProcessor=\"";
 const char* _OptimizeForWindows98 		= "\n\t\t\t\tOptimizeForWindows98=\"";
@@ -345,6 +346,7 @@ VCCLCompilerTool::VCCLCompilerTool()
 	KeepComments( unset ),
 	MinimalRebuild( unset ),
 	OmitFramePointers( unset ),
+	OpenMP( unset ),
 	Optimization( optimizeCustom ),
 	OptimizeForProcessor( procOptimizeBlended ),
 	OptimizeForWindowsApplication( unset ),
@@ -416,6 +418,7 @@ QTextStream &operator<<( QTextStream &strm, const VCCLCompilerTool &tool )
     if ( !tool.ProgramDataBaseFileName.isNull() ) strm << _ProgramDataBaseFileName << tool.ProgramDataBaseFileName.latin1() << "\"";
     if ( tool.RuntimeLibrary != rtUnknown ) strm << EPair( _RuntimeLibrary, tool.RuntimeLibrary );
     strm << TPair( _RuntimeTypeInfo, tool.RuntimeTypeInfo );
+    strm << TPair( _OpenMP, tool.OpenMP );
     strm << TPair( _ShowIncludes, tool.ShowIncludes );
     strm << TPair( _SmallerTypeCheck, tool.SmallerTypeCheck );
     strm << TPair( _StringPooling, tool.StringPooling );
@@ -908,6 +911,12 @@ bool VCCLCompilerTool::parseOption( const char* option )
 	    break;
 	}
 	found = FALSE; break;
+    case 'o':
+        if ( second == 'p' && third == 'e' && fourth == 'n' ) {
+            OpenMP = _True;
+            break;
+        }
+	found = FALSE; break;
     case 's':
 	if ( second == 'h' && third == 'o' && fourth == 'w' ) {
 	    ShowIncludes = _True;
