@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qcursor_x11.cpp   3.3.4   edited Jan 21 2004 $
+** $Id: qt/qcursor_x11.cpp   3.3.6   edited Jan 30 09:42 $
 **
 ** Implementation of QCursor class for X11
 **
 ** Created : 940219
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -252,6 +252,8 @@ void QCursor::setBitmap( const QBitmap &bitmap, const QBitmap &mask,
     data->bg.red   = 255 << 8;
     data->bg.green = 255 << 8;
     data->bg.blue  = 255 << 8;
+    update(); // Xcursor's backward compatibility hack needs the cursor to be created
+              // right after the bitmaps are created and filled with data
 }
 
 
