@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qkeysequence.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qkeysequence.cpp   3.3.6   edited Feb 16 10:45 $
 **
 ** Implementation of QKeySequence class
 **
 ** Created : 0108007
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -363,6 +363,8 @@ int QKeySequence::assign( QString keyseq )
 	    if ( ' ' == keyseq[p+1] ) { // Space after comma
 		diff = 1;
 		p++;
+	    } else if ( '\0' == keyseq[p+1] ) { // Last comma 'Ctrl+,'
+                p = -1;
 	    } else {
 		diff = 0;
 	    }
@@ -433,7 +435,7 @@ int QKeySequence::decodeString( const QString& str )
 	else
 	    ret |= accel[0].upper().unicode();
 	ret |= UNICODE_ACCEL;
-    } else if ( accel[0] == 'F' && (fnum = accel.mid(1).toInt()) ) {
+    } else if ( accel[0] == 'F' && (fnum = accel.mid(1).toInt()) && (fnum >= 1) && (fnum <= 35) ) {
         ret |= Key_F1 + fnum - 1;
     } else {
 	// Check through translation table for the correct key name
@@ -463,58 +465,39 @@ int QKeySequence::decodeString( const QString& str )
  */
 QString QKeySequence::encodeString( int key )
 {
-    bool plus_next = TRUE;
     QString s;
+#if defined(Q_OS_MAC) && !defined(QWS)
+    // On MAC the order is Meta, Alt, Shift, Control.
+    if ( (key & META) == META )
+	s += QMAC_META;
+    if ( (key & ALT) == ALT )
+	s += QMAC_ALT;
+    if ( (key & SHIFT) == SHIFT )
+	s += QMAC_SHIFT;
+    if ( (key & CTRL) == CTRL )
+	s += QMAC_CTRL;
+#else
+    // On other systems the order is Meta, Control, Alt, Shift
+    if ( (key & META) == META )
+	s += QAccel::tr( "Meta" );
     if ( (key & CTRL) == CTRL ) {
-	plus_next = TRUE;
-	QString ctrl = QAccel::tr( "Ctrl" );
-#ifdef QMAC_CTRL
-	if(ctrl == "Ctrl") {
-	    plus_next = FALSE;
-	    ctrl = QMAC_CTRL;
-	}
-#endif
-	s += ctrl;
+	if ( !s.isEmpty() )
+	    s += QAccel::tr( "+" );
+	s += QAccel::tr( "Ctrl" );
     }
     if ( (key & ALT) == ALT ) {
-	if ( plus_next && !s.isEmpty() )
+	if ( !s.isEmpty() )
 	    s += QAccel::tr( "+" );
-	plus_next = TRUE;
-	QString alt = QAccel::tr( "Alt" );
-#ifdef QMAC_ALT
-	if(alt == "Alt") {
-	    plus_next = FALSE;
-	    alt = QMAC_ALT;
-	}
-#endif
-	s += alt;
-    }
-    if ( (key & META) == META ) {
-	if ( plus_next && !s.isEmpty() )
-	    s += QAccel::tr( "+" );
-	plus_next = TRUE;
-	QString meta = QAccel::tr( "Meta" ); 
-#ifdef QMAC_META
-	if(meta == "Meta") {
-	    plus_next = FALSE;
-	    meta = QMAC_META;
-	}
-#endif
-	s += meta;
+	s += QAccel::tr( "Alt" );
     }
     if ( (key & SHIFT) == SHIFT ) {
-	if ( plus_next && !s.isEmpty() )
+	if ( !s.isEmpty() )
 	    s += QAccel::tr( "+" );
-	plus_next = TRUE;
-	QString shift = QAccel::tr( "Shift" );
-#ifdef QMAC_SHIFT
-	if(shift == "Shift") {
-	    plus_next = FALSE;
-	    shift = QMAC_SHIFT;
-	}
-#endif
-	s += shift;
+	s += QAccel::tr( "Shift" );
     }
+#endif
+
+
     key &= ~(SHIFT | CTRL | ALT | META );
     QString p;
 
@@ -545,13 +528,13 @@ QString QKeySequence::encodeString( int key )
 	    // Note: This character should NOT be upper()'ed, see above!
 	    p = QChar(key & 0xffff);
     }
-    if ( s.isEmpty() ) {
-	s = p;
-    } else {
-	if(plus_next)
-	    s += QAccel::tr( "+" );
-	s += p;
-    }
+
+#ifndef Q_OS_MAC
+    if ( !s.isEmpty() )
+	s += QAccel::tr( "+" );
+#endif
+
+    s += p;
     return s;
 }
 
@@ -592,6 +575,9 @@ Qt::SequenceMatch QKeySequence::matches( const QKeySequence& seq ) const
     "Alt+X, Ctrl+Y, Z". The strings, "Ctrl", "Shift", etc. are
     translated (using QObject::tr()) in the "QAccel" scope. If the key
     sequence has no keys, QString::null is returned.
+
+    On Mac OS X, the string returned resembles the sequence that is shown in
+    the menubar.
 */
 QKeySequence::operator QString() const
 {
