@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qwmatrix.cpp   3.2.3   edited Oct 13 17:17 $
+** $Id: qt/qwmatrix.cpp   3.3.6   edited Feb 9 13:21 $
 **
 ** Implementation of QWMatrix class
 **
 ** Created : 941020
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -723,53 +723,32 @@ QRegion QWMatrix::operator * (const QRegion &r ) const
     QRegion result;
     register QRect *rect = rects.data();
     register int i = rects.size();
-    if ( qt_old_transformations ) {
-	if ( _m12 == 0.0F && _m21 == 0.0F ) {
-	    // simple case, no rotation
-	    while ( i ) {
-		*rect = QRect( map(rect->topLeft()), map(rect->bottomRight()) ).normalize();
-		rect++;
-		i--;
+    if ( _m12 == 0.0F && _m21 == 0.0F && _m11 > 1.0F && _m22 > 1.0F ) {
+	// simple case, no rotation
+	while ( i ) {
+	    int x = qRound( _m11*rect->x() + _dx );
+	    int y = qRound( _m22*rect->y() + _dy );
+	    int w = qRound( _m11*rect->width() );
+	    int h = qRound( _m22*rect->height() );
+	    if ( w < 0 ) {
+		w = -w;
+		x -= w-1;
 	    }
-	    result.setRects( rects.data(), rects.size() );
-	} else {
-	    while ( i ) {
-		QPointArray a( *rect );
-		a = map( a );
-		result |= QRegion( a );
-		rect++;
-		i--;
+	    if ( h < 0 ) {
+		h = -h;
+		y -= h-1;
 	    }
+	    *rect = QRect( x, y, w, h );
+	    rect++;
+	    i--;
 	}
+	result.setRects( rects.data(), rects.size() );
     } else {
-	if ( _m12 == 0.0F && _m21 == 0.0F ) {
-	    // simple case, no rotation
-	    while ( i ) {
-		int x = qRound( _m11*rect->x() + _dx );
-		int y = qRound( _m22*rect->y() + _dy );
-		int w = qRound( _m11*rect->width() );
-		int h = qRound( _m22*rect->height() );
-		if ( w < 0 ) {
-		    w = -w;
-		    x -= w-1;
-		}
-		if ( h < 0 ) {
-		    h = -h;
-		    y -= h-1;
-		}
-		*rect = QRect( x, y, w, h );
-		rect++;
-		i--;
-	    }
-	    result.setRects( rects.data(), rects.size() );
-	} else {
-	    while ( i ) {
-		result |= operator *( *rect );
-		rect++;
-		i--;
-	    }
+	while ( i ) {
+	    result |= operator *( *rect );
+	    rect++;
+	    i--;
 	}
-
     }
     return result;
 }
