@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qhostaddress.cpp   3.2.3   edited Aug 12 16:24 $
+** $Id: qt/qhostaddress.cpp   3.3.6   edited Aug 31 2005 $
 **
 ** Implementation of QHostAddress class.
 **
 ** Created : 979899
 **
-** Copyright (C) 1997-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1997-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -46,6 +46,7 @@ public:
     {
     }
     QHostAddressPrivate( Q_UINT8 *a_ );
+    QHostAddressPrivate(const Q_IPV6ADDR &a_);
     ~QHostAddressPrivate()
     {
     }
@@ -54,27 +55,30 @@ public:
     {
 	a = from.a;
 	isIp4 = from.isIp4;
-	for ( int i=0; i<16; i++ ) {
-	    a6[i] = from.a6[i];
-	}
+	a6 = from.a6;
 	return *this;
     }
 
 private:
     Q_UINT32 a;     // ip 4 address
-    Q_UINT8 a6[16]; // ip 6 address
+    Q_IPV6ADDR a6; // ip 6 address
     bool isIp4;
 
     friend class QHostAddress;
 };
 
-QHostAddressPrivate::QHostAddressPrivate( Q_UINT8 *a_ ) : a(0), isIp4(FALSE)
+QHostAddressPrivate::QHostAddressPrivate(Q_UINT8 *a_) : a(0), isIp4(FALSE)
 {
     for ( int i=0; i<16; i++ ) {
-	a6[i] = a_[i];
+	a6.c[i] = a_[i];
     }
 }
 
+QHostAddressPrivate::QHostAddressPrivate(const Q_IPV6ADDR &a_) : a(0), isIp4(FALSE)
+{
+    a6 = a_;
+}
+
 /*!
     \class QHostAddress qhostaddress.h
     \brief The QHostAddress class provides an IP address.
@@ -87,8 +91,7 @@ QHostAddressPrivate::QHostAddressPrivate( Q_UINT8 *a_ ) : a(0), isIp4(FALSE)
 
     This class contains an IP address in a platform and protocol
     independent manner. It stores both IPv4 and IPv6 addresses in a
-    way that you can easily access on any platform. (Qt does not
-    currently provide IPv6 support.)
+    way that you can easily access on any platform.
 
     QHostAddress is normally used with the classes QSocket,
     QServerSocket and QSocketDevice to set up a server or to connect
@@ -130,6 +133,24 @@ QHostAddress::QHostAddress( Q_UINT8 *ip6Addr )
 {
 }
 
+/*!
+    Creates a host address object with the IPv6 address, \a ip6Addr.
+*/
+QHostAddress::QHostAddress(const Q_IPV6ADDR &ip6Addr)
+    : d(new QHostAddressPrivate(ip6Addr))
+{
+}
+
+// ### DOC: Can only make this public if we specify precisely the
+// format of the address string.
+/*!
+    \internal
+*/
+QHostAddress::QHostAddress(const QString &address)
+    : d( new QHostAddressPrivate )
+{
+    setAddress( address );
+}
 
 /*!
     Creates a copy of \a address.
@@ -186,6 +207,26 @@ void QHostAddress::setAddress( Q_UINT8 *ip6Addr )
 }
 
 #ifndef QT_NO_STRINGLIST
+static bool parseIp4(const QString& address, Q_UINT32 *addr)
+{
+    QStringList ipv4 = QStringList::split(".", address, FALSE);
+    if (ipv4.count() == 4) {
+	int i = 0;
+	bool ok = TRUE;
+	while(ok && i < 4) {
+	    uint byteValue = ipv4[i].toUInt(&ok);
+	    if (byteValue > 255)
+		ok = FALSE;
+	    if (ok)
+		*addr = (*addr << 8) + byteValue;
+	    ++i;
+	}
+	if (ok)
+	    return TRUE;
+    }
+    return FALSE;
+}
+
 /*!
     \overload
 
@@ -194,95 +235,111 @@ void QHostAddress::setAddress( Q_UINT8 *ip6Addr )
     sets the address if the address was successfully parsed; otherwise
     returns FALSE and leaves the address unchanged.
 */
-bool QHostAddress::setAddress( const QString& address )
+bool QHostAddress::setAddress(const QString& address)
 {
     QString a = address.simplifyWhiteSpace();
 
     // try ipv4
-    QStringList ipv4 = QStringList::split( ".", a, FALSE );
-    if ( ipv4.count() == 4 ) {
-	Q_UINT32 maybe = 0;
-	int i = 0;
-	bool ok = TRUE;
-	while( ok && i < 4 ) {
-	    uint byteValue = ipv4[i].toUInt( &ok );
-	    if ( byteValue > 255 )
-		ok = FALSE;
-	    if ( ok )
-		maybe = maybe * 256 + byteValue;
-	    i++;
-	}
-	if ( ok ) {
-	    setAddress( maybe );
-	    return TRUE;
-	}
+    Q_UINT32 maybeIp4 = 0;
+    if (parseIp4(address, &maybeIp4)) {
+	setAddress(maybeIp4);
+	return TRUE;
     }
 
-#if 0
-    // ### add ipv6 support???
     // try ipv6
-    QStringList ipv6 = QStringList::split( ":", a, TRUE );
-    int count = ipv6.count();
-    if ( count < 3 )
+    QStringList ipv6 = QStringList::split(":", a, TRUE);
+    int count = (int)ipv6.count();
+    if (count < 3)
 	return FALSE; // there must be at least two ":"
-    if ( count > 8 )
+    if (count > 8)
 	return FALSE; // maximum of seven ":" exceeded
-    Q_UINT8 maybe[16];
+    Q_UINT8 maybeIp6[16];
     int mc = 16;
-    bool colcolFound = FALSE;
-    for ( int i=count-1; i>=0; i-- ) {
-	if ( ipv6[i].isEmpty() ) {
-	    if ( i==count-1 ) {
+    int fillCount = 9 - count;
+    for (int i=count-1; i>=0; --i) {
+	if ( mc <= 0 )
+	    return FALSE;
+
+	if (ipv6[i].isEmpty()) {
+	    if (i==count-1) {
 		// special case: ":" is last character
-		if ( !ipv6[i-1].isEmpty() )
+		if (!ipv6[i-1].isEmpty())
 		    return FALSE;
-	    } else if ( i==0 ) {
+		maybeIp6[--mc] = 0;
+		maybeIp6[--mc] = 0;
+	    } else if (i==0) {
 		// special case: ":" is first character
-		if ( !ipv6[i+1].isEmpty() )
+		if (!ipv6[i+1].isEmpty())
 		    return FALSE;
-		maybe[--mc] = 0;
-		maybe[--mc] = 0;
+		maybeIp6[--mc] = 0;
+		maybeIp6[--mc] = 0;
 	    } else {
-		// special case: found a "::" somewhere
-		if ( colcolFound )
-		    return FALSE; // more then one "::"
-		colcolFound = TRUE;
-		// ### add the right numbers of zeros
+		for (int j=0; j<fillCount; ++j) {
+		    if ( mc <= 0 )
+			return FALSE;
+		    maybeIp6[--mc] = 0;
+		    maybeIp6[--mc] = 0;
+		}
 	    }
 	} else {
 	    bool ok = FALSE;
-	    uint byteValue = ipv6[i].toUInt( &ok, 16 );
-	    if ( ok && byteValue < 0xffff ) {
-		maybe[--mc] = byteValue / 256;
-		maybe[--mc] = byteValue % 256;
+	    uint byteValue = ipv6[i].toUInt(&ok, 16);
+	    if (ok && byteValue <= 0xffff) {
+		maybeIp6[--mc] = byteValue & 0xff;
+		maybeIp6[--mc] = (byteValue >> 8) & 0xff;
 	    } else {
-		if ( i == count-1 ) {
-		    // ### parse the ipv4 part of a mixed type
+		if (i == count-1) {
+		    // parse the ipv4 part of a mixed type
+		    if (!parseIp4(ipv6[i], &maybeIp4))
+			return FALSE;
+		    maybeIp6[--mc] = maybeIp4 & 0xff;
+		    maybeIp6[--mc] = (maybeIp4 >> 8) & 0xff;
+		    maybeIp6[--mc] = (maybeIp4 >> 16) & 0xff;
+		    maybeIp6[--mc] = (maybeIp4 >> 24) & 0xff;
+		    --fillCount;
 		} else {
-		    break;
+		    return FALSE;
 		}
 	    }
 	}
     }
-    if ( mc == 0 ) {
-	setAddress( maybe );
+    if (mc == 0) {
+	setAddress(maybeIp6);
 	return TRUE;
     }
-#endif
 
     return FALSE;
 }
 #endif
 
+/*!
+    \obsolete
+
+    Use isIPv4Address() instead.
+*/
+bool QHostAddress::isIp4Addr() const
+{
+    return isIPv4Address();
+}
+
 /*!
     Returns TRUE if the host address represents an IPv4 address;
     otherwise returns FALSE.
 */
-bool QHostAddress::isIp4Addr() const
+bool QHostAddress::isIPv4Address() const
 {
     return d->isIp4;
 }
 
+/*!
+    \obsolete
+
+    Use toIPv4Address() instead.
+*/
+Q_UINT32 QHostAddress::ip4Addr() const
+{
+    return toIPv4Address();
+}
 
 /*!
     Returns the IPv4 address as a number.
@@ -294,11 +351,42 @@ bool QHostAddress::isIp4Addr() const
 
     \sa toString()
 */
-Q_UINT32 QHostAddress::ip4Addr() const
+Q_UINT32 QHostAddress::toIPv4Address() const
 {
     return d->a;
 }
 
+/*!
+    Returns TRUE if the host address represents an IPv6 address;
+    otherwise returns FALSE.
+*/
+bool QHostAddress::isIPv6Address() const
+{
+    return !d->isIp4;
+}
+
+/*!
+    Returns the IPv6 address as a Q_IPV6ADDR structure. The structure
+    consists of 16 unsigned characters.
+
+    \code
+        Q_IPV6ADDR addr = hostAddr.ip6Addr();
+        // addr.c[] contains 16 unsigned characters
+
+        for (int i = 0; i < 16; ++i) {
+            // process addr.c[i]
+        }
+    \endcode
+
+    This value is only valid when isIPv6Address() returns TRUE.
+
+    \sa toString()
+*/
+Q_IPV6ADDR QHostAddress::toIPv6Address() const
+{
+    return d->a6;
+}
+
 #ifndef QT_NO_SPRINTF
 /*!
     Returns the address as a string.
@@ -319,11 +407,11 @@ QString QHostAddress::toString() const
     } else {
 	Q_UINT16 ugle[8];
 	for ( int i=0; i<8; i++ ) {
-	    ugle[i] = ( (Q_UINT16)( d->a6[2*i] ) << 8 ) |
-		( (Q_UINT16)( d->a6[2*i+1] ) );
+	    ugle[i] = ( (Q_UINT16)( d->a6.c[2*i] ) << 8 ) |
+		( (Q_UINT16)( d->a6.c[2*i+1] ) );
 	}
 	QString s;
-	s.sprintf( "%x:%x:%x:%x:%x:%x:%x:%x",
+	s.sprintf( "%X:%X:%X:%X:%X:%X:%X:%X",
 		ugle[0], ugle[1], ugle[2], ugle[3],
 		ugle[4], ugle[5], ugle[6], ugle[7] );
 	return s;
@@ -353,7 +441,7 @@ bool QHostAddress::isNull() const
 	return d->a == 0;
     int i = 0;
     while( i < 16 ) {
-	if ( d->a6[i++] != 0 )
+	if ( d->a6.c[i++] != 0 )
 	    return FALSE;
     }
     return TRUE;
