@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qhttp.cpp   3.2.3   edited Aug 12 16:24 $
+** $Id: qt/qhttp.cpp   3.3.6   edited Jan 26 15:17 $
 **
 ** Implementation of QHttp and related classes.
 **
 ** Created : 970521
 **
-** Copyright (C) 1997-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1997-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -840,6 +840,9 @@ QString QHttpResponseHeader::toString() const
     It is part of the <a href="commercialeditions.html">Qt Enterprise Edition</a>.
 \endif
     HTTP.
+\if defined(commercial_edition)
+    It is part of the <a href="commercialeditions.html">Qt Enterprise Edition</a>.
+\endif
 
     \ingroup io
     \module network
@@ -1227,10 +1230,10 @@ void QHttp::init()
 	    this, SLOT( slotClosed() ) );
     connect( &d->socket, SIGNAL( readyRead() ),
 	    this, SLOT( slotReadyRead() ) );
-    connect( &d->socket, SIGNAL( error( int ) ),
-	    this, SLOT( slotError( int ) ) );
-    connect( &d->socket, SIGNAL( bytesWritten( int ) ),
-	    this, SLOT( slotBytesWritten( int ) ) );
+    connect( &d->socket, SIGNAL( error(int) ),
+	    this, SLOT( slotError(int) ) );
+    connect( &d->socket, SIGNAL( bytesWritten(int) ),
+	    this, SLOT( slotBytesWritten(int) ) );
 
     d->idleTimer = startTimer( 0 );
 }
@@ -1803,7 +1806,8 @@ void QHttp::sendRequest()
 
     // Do we need to setup a new connection or can we reuse an
     // existing one ?
-    if ( d->socket.peerName() != d->hostname || d->socket.state() != QSocket::Connection ) {
+    if ( d->socket.peerName() != d->hostname || d->socket.peerPort() != d->port
+        || d->socket.state() != QSocket::Connection ) {
 	setState( QHttp::Connecting );
 	d->socket.connectToHost( d->hostname, d->port );
     } else {
@@ -1943,7 +1947,7 @@ void QHttp::slotReadyRead()
 	d->chunkedSize = -1;
     }
 
-    if ( d->readHeader ) {
+    while ( d->readHeader ) {
 	bool end = FALSE;
 	QString tmp;
 	while ( !end && d->socket.canReadLine() ) {
@@ -1954,26 +1958,34 @@ void QHttp::slotReadyRead()
 		d->headerStr += tmp;
 	}
 
-	if ( end ) {
-#if defined(QHTTP_DEBUG)
-	    qDebug( "QHttp: read response header:\n---{\n%s}---", d->headerStr.latin1() );
-#endif
-	    d->readHeader = FALSE;
-	    d->response = QHttpResponseHeader( d->headerStr );
-	    d->headerStr = "";
-#if defined(QHTTP_DEBUG)
-	    qDebug( "QHttp: read response header:\n---{\n%s}---", d->response.toString().latin1() );
-#endif
-	    // Check header
-	    if ( !d->response.isValid() ) {
-		finishedWithError( tr("Invalid HTTP response header"), InvalidResponseHeader );
-		close();
-		return;
-	    }
-	    if ( d->response.hasKey( "transfer-encoding" ) && d->response.value( "transfer-encoding" ).contains( "chunked" ) )
-		d->chunkedSize = 0;
-	    emit responseHeaderReceived( d->response );
+	if ( !end )
+	    return;
 
+#if defined(QHTTP_DEBUG)
+	qDebug( "QHttp: read response header:\n---{\n%s}---", d->headerStr.latin1() );
+#endif
+	d->response = QHttpResponseHeader( d->headerStr );
+	d->headerStr = "";
+#if defined(QHTTP_DEBUG)
+	qDebug( "QHttp: read response header:\n---{\n%s}---", d->response.toString().latin1() );
+#endif
+	// Check header
+	if ( !d->response.isValid() ) {
+	    finishedWithError( tr("Invalid HTTP response header"), InvalidResponseHeader );
+	    close();
+	    return;
+	}
+
+	// The 100-continue header is ignored, because when using the
+	// POST method, we send both the request header and data in
+	// one chunk.
+	if (d->response.statusCode() != 100) {
+	    d->readHeader = FALSE;
+	    if ( d->response.hasKey( "transfer-encoding" ) &&
+		 d->response.value( "transfer-encoding" ).lower().contains( "chunked" ) )
+		d->chunkedSize = 0;
+
+	    emit responseHeaderReceived( d->response );
 	}
     }
 
@@ -2031,6 +2043,7 @@ void QHttp::slotReadyRead()
 
 		    // read data
 		    uint toRead = QMIN( (Q_LONG)n, d->chunkedSize );
+		    // uint toRead = QMIN( (Q_LONG)n, (d->chunkedSize < 0 ? (Q_LONG)n : d->chunkedSize) );
 		    if ( !arr )
 			arr = new QByteArray( 0 );
 		    uint oldArrSize = arr->size();
@@ -2095,7 +2108,7 @@ void QHttp::slotReadyRead()
 
 	if ( everythingRead ) {
 	    // Handle "Connection: close"
-	    if ( d->response.value("connection") == "close" ) {
+	    if ( d->response.value("connection").lower() == "close" ) {
 		close();
 	    } else {
 		setState( Connected );
