@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qsocket.cpp   3.3.4   edited Jan 3 16:26 $
+** $Id: qt/qsocket.cpp   3.3.6   edited Sep 15 12:11 $
 **
 ** Implementation of QSocket class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -233,6 +233,9 @@ void QSocketPrivate::setSocketDevice( QSocket *q, QSocketDevice *device )
     It provides a totally non-blocking QIODevice, and modifies and
     extends the API of QIODevice with socket-specific code.
 
+    Note that a QApplication must have been constructed before this
+    class can be used.
+
     The functions you're likely to call most are connectToHost(),
     bytesAvailable(), canReadLine() and the ones it inherits from
     QIODevice.
@@ -260,8 +263,8 @@ void QSocketPrivate::setSocketDevice( QSocket *q, QSocketDevice *device )
     the IP address and port used for the connection. The peerAddress()
     and peerPort() functions return the IP address and port used by
     the peer, and peerName() returns the name of the peer (normally
-    the name that was passed to connectToHost()). socket() returns a
-    pointer to the QSocketDevice used for this socket.
+    the name that was passed to connectToHost()). socketDevice()
+    returns a pointer to the QSocketDevice used for this socket.
 
     QSocket inherits QIODevice, and reimplements some functions. In
     general, you can treat it as a QIODevice for writing, and mostly
@@ -277,6 +280,11 @@ void QSocketPrivate::setSocketDevice( QSocket *q, QSocketDevice *device )
     \warning QSocket is not suitable for use in threads. If you need
     to uses sockets in threads use the lower-level QSocketDevice class.
 
+    \warning Because Qt doesn't use the native socketstream
+    implementation on Mac OS X, QSocket has an implicit transfer
+    latency of 100ms. You can achieve lower latency on Mac OS X by
+    using QSocketDevice instead.
+
     \sa QSocketDevice, QHostAddress, QSocketNotifier
 */
 
@@ -286,6 +294,9 @@ void QSocketPrivate::setSocketDevice( QSocket *q, QSocketDevice *device )
 
     The \a parent and \a name arguments are passed on to the QObject
     constructor.
+
+    Note that a QApplication must have been constructed before sockets
+    can be used.
 */
 
 QSocket::QSocket( QObject *parent, const char *name )
