@@ -4,7 +4,7 @@
 **
 ** Created : 001103
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the sql module of the Qt GUI Toolkit.
 **
@@ -35,25 +35,30 @@
 **********************************************************************/
 
 #include "qsql_psql.h"
+#include <private/qsqlextension_p.h>
 
+#include <math.h>
+
+#include <qpointarray.h>
 #include <qsqlrecord.h>
 #include <qregexp.h>
 #include <qdatetime.h>
-#include <qpointarray.h>
-#undef DEBUG // the PostgreSQL headers redefines this
+// PostgreSQL header <utils/elog.h> included by <postgres.h> redefines DEBUG.
+#if defined(DEBUG)
+# undef DEBUG
+#endif
 #include <postgres.h>
 #include <libpq/libpq-fs.h>
-#if defined(Q_CC_MSVC)
-#pragma warning(disable: 4273) // '_errno' : inconsistent dll linkage.  dllexport assumed
+// PostgreSQL header <catalog/pg_type.h> redefines errno erroneously.
+#if defined(errno)
+# undef errno
 #endif
+#define errno qt_psql_errno
 #include <catalog/pg_type.h>
-#if defined(Q_CC_MSVC)
-#pragma warning(default: 4273)
-#endif
-#ifndef Q_WS_MAC
-#include <utils/geo_decls.h>
-#endif
-#include <math.h>
+#undef errno
+
+QPtrDict<QSqlDriverExtension> *qSqlDriverExtDict();
+QPtrDict<QSqlOpenExtension> *qSqlOpenExtDict();
 
 class QPSQLPrivate
 {
@@ -64,12 +69,58 @@ public:
     bool        isUtf8;
 };
 
-QSqlError qMakeError( const QString& err, int type, const QPSQLPrivate* p )
+class QPSQLDriverExtension : public QSqlDriverExtension
 {
-    return QSqlError("QPSQL: " + err, QString(PQerrorMessage( p->connection )), type);
+public:
+    QPSQLDriverExtension( QPSQLDriver *dri )
+	: QSqlDriverExtension(), driver(dri) { }
+    ~QPSQLDriverExtension() {}
+
+    bool isOpen() const;
+private:
+    QPSQLDriver *driver;
+};
+
+bool QPSQLDriverExtension::isOpen() const
+{
+    return PQstatus( driver->connection() ) == CONNECTION_OK;
 }
 
-QVariant::Type qDecodePSQLType( int t )
+class QPSQLOpenExtension : public QSqlOpenExtension
+{
+public:
+    QPSQLOpenExtension( QPSQLDriver *dri )
+	: QSqlOpenExtension(), driver(dri) { }
+    ~QPSQLOpenExtension() {}
+
+    bool open( const QString& db,
+	       const QString& user,
+	       const QString& password,
+	       const QString& host,
+	       int port,
+	       const QString& connOpts );
+private:
+    QPSQLDriver *driver;
+};
+
+bool QPSQLOpenExtension::open( const QString& db,
+			       const QString& user,
+			       const QString& password,
+			       const QString& host,
+			       int port,
+			       const QString& connOpts )
+{
+    return driver->open( db, user, password, host, port, connOpts );
+}
+
+static QSqlError qMakeError( const QString& err, int type, const QPSQLPrivate* p )
+{
+    const char *s = PQerrorMessage(p->connection);
+    QString msg = p->isUtf8 ? QString::fromUtf8(s) : QString::fromLocal8Bit(s);
+    return QSqlError("QPSQL: " + err, msg, type);
+}
+
+static QVariant::Type qDecodePSQLType( int t )
 {
     QVariant::Type type = QVariant::Invalid;
     switch ( t ) {
@@ -77,6 +128,8 @@ QVariant::Type qDecodePSQLType( int t )
 	type = QVariant::Bool;
 	break;
     case INT8OID	:
+	type = QVariant::LongLong;
+	break;
     case INT2OID	:
 	//    case INT2VECTOROID  : // 7.x
     case INT4OID        :
@@ -93,7 +146,9 @@ QVariant::Type qDecodePSQLType( int t )
 	type = QVariant::Date;
 	break;
     case TIMEOID	:
-	//    case TIMETZOID      : // 7.x
+#ifdef TIMETZOID // 7.x
+	case TIMETZOID  :
+#endif
 	type = QVariant::Time;
 	break;
     case TIMESTAMPOID   :
@@ -109,18 +164,10 @@ QVariant::Type qDecodePSQLType( int t )
 #endif
 	type = QVariant::DateTime;
 	break;
-    case POINTOID       :
-	type = QVariant::Point;
-	break;
-    case BOXOID         :
-	type = QVariant::Rect;
-	break;
-    case POLYGONOID     :
-	type = QVariant::PointArray;
-	break;
 	//    case ZPBITOID	: // 7.x
 	//    case VARBITOID	: // 7.x
     case OIDOID         :
+    case BYTEAOID       :
 	type = QVariant::ByteArray;
 	break;
     case REGPROCOID     :
@@ -129,13 +176,7 @@ QVariant::Type qDecodePSQLType( int t )
     case CIDOID         :
 	//    case OIDVECTOROID   : // 7.x
     case UNKNOWNOID     :
-    case INETOID        :
-    case CIDROID        :
 	//    case TINTERVALOID   : // 7.x
-    case CIRCLEOID      :
-    case PATHOID        :
-    case LSEGOID        :
-    case LINEOID        :
 	type = QVariant::Invalid;
 	break;
     default:
@@ -145,20 +186,16 @@ QVariant::Type qDecodePSQLType( int t )
     case VARCHAROID	:
     case TEXTOID	:
     case NAMEOID	:
-    case BYTEAOID	:
     case CASHOID        :
+    case INETOID        :
+    case CIDROID        :
+    case CIRCLEOID      :
 	type = QVariant::String;
 	break;
     }
     return type;
 }
 
-QVariant::Type qFieldType( QPSQLPrivate* p, int i )
-{
-    QVariant::Type type = qDecodePSQLType( PQftype( p->result, i ) );
-    return type;
-}
-
 QPSQLResult::QPSQLResult( const QPSQLDriver* db, const QPSQLPrivate* p )
 : QSqlResult( db ),
   currentSize( 0 )
@@ -213,10 +250,10 @@ bool QPSQLResult::fetchLast()
 }
 
 // some Postgres conversions
-QPoint pointFromString( const QString& s)
+static QPoint pointFromString( const QString& s)
 {
     // format '(x,y)'
-    int pivot = s.find( QRegExp(",") );
+    int pivot = s.find( ',' );
     if ( pivot != -1 ) {
 	int x = s.mid( 1, pivot-1 ).toInt();
 	int y = s.mid( pivot+1, s.length()-pivot-2 ).toInt();
@@ -225,31 +262,21 @@ QPoint pointFromString( const QString& s)
 	return QPoint();
 }
 
-QDate qDateFromUInt( uint dt )
-{
-    int y,m,d;
-    QDate::julianToGregorian( dt, y, m, d );
-    return QDate( y, m, d );
-}
-
-/* // ### this should be obsolete?
-QTime qTimeFromDouble( double tm )
-{
-    int hour = ((int)tm / ( 60 * 60 ) );
-    int min = (((int) (tm / 60)) % 60 );
-    int sec = (((int) tm) % 60 );
-    return QTime( hour, min, sec );
-}
-*/
-
 QVariant QPSQLResult::data( int i )
 {
-    QVariant::Type type = qFieldType( d, i );
-    QString val;
-    if ( d->isUtf8 ) {
-	val = QString::fromUtf8( PQgetvalue( d->result, at(), i ) );
-    } else {
-	val = QString::fromLocal8Bit( PQgetvalue( d->result, at(), i ) );
+    if ( i >= PQnfields( d->result ) ) {
+	qWarning( "QPSQLResult::data: column %d out of range", i );
+	return QVariant();
+    }
+    int ptype = PQftype( d->result, i );
+    QVariant::Type type = qDecodePSQLType( ptype );
+    const QString val = ( d->isUtf8 && ptype != BYTEAOID ) ?
+			QString::fromUtf8( PQgetvalue( d->result, at(), i ) ) :
+			QString::fromLocal8Bit( PQgetvalue( d->result, at(), i ) );
+    if ( PQgetisnull( d->result, at(), i ) ) {
+	QVariant v;
+	v.cast( type );
+	return v;
     }
     switch ( type ) {
     case QVariant::Bool:
@@ -258,37 +285,53 @@ QVariant QPSQLResult::data( int i )
 	    return ( b );
 	}
     case QVariant::String:
-	if ( PQgetisnull( d->result, at(), i ) ) {
-	    return QVariant( QString() );
-	} else {
-	    return QVariant( val );
-	}
+	return QVariant( val );
+    case QVariant::LongLong:
+	if ( val[0] == '-' )
+	    return QVariant( val.toLongLong() );
+	else
+	    return QVariant( val.toULongLong() );
     case QVariant::Int:
 	return QVariant( val.toInt() );
     case QVariant::Double:
+	if ( ptype == NUMERICOID )
+	    return QVariant( val );
 	return QVariant( val.toDouble() );
     case QVariant::Date:
-	return QVariant( QDate::fromString( val, Qt::ISODate ) );
+	if ( val.isEmpty() ) {
+	    return QVariant( QDate() );
+	} else {
+	    return QVariant( QDate::fromString( val, Qt::ISODate ) );
+	}
     case QVariant::Time:
+	if ( val.isEmpty() )
+	    return QVariant( QTime() );
+	if ( val.at( val.length() - 3 ) == '+' )
+	    // strip the timezone
+	    return QVariant( QTime::fromString( val.left( val.length() - 3 ), Qt::ISODate ) );
 	return QVariant( QTime::fromString( val, Qt::ISODate ) );
-    case QVariant::DateTime:
+    case QVariant::DateTime: {
 	if ( val.length() < 10 )
 	    return QVariant( QDateTime() );
 	// remove the timezone
-	if ( val.find( "+", val.length() - 3 ) >= 0 )
-	    val = val.left( val.length() - 3 );
-	// for some reasons the milliseconds are sometimes only 2 digits
-	// (12 instead of 120)
-	if ( val.find( ".", val.length() - 3 ) >= 0 )
-	    val += "0";
-	return QVariant( QDateTime::fromString( val, Qt::ISODate ) );
+	QString dtval = val;
+	if ( dtval.at( dtval.length() - 3 ) == '+' )
+	    dtval.truncate( dtval.length() - 3 );
+	// milliseconds are sometimes returned with 2 digits only
+	if ( dtval.at( dtval.length() - 3 ).isPunct() )
+	    dtval += '0';
+	if ( dtval.isEmpty() )
+	    return QVariant( QDateTime() );
+	else
+	    return QVariant( QDateTime::fromString( dtval, Qt::ISODate ) );
+    }
     case QVariant::Point:
 	return QVariant( pointFromString( val ) );
     case QVariant::Rect: // format '(x,y),(x',y')'
 	{
-	    int pivot = val.find( QRegExp( "\\),\\(" ) );
+	    int pivot = val.find( "),(" );
 	    if ( pivot != -1 )
-		return QVariant( QRect( pointFromString( val.mid(0,pivot+1) ), pointFromString( val.mid(pivot+2,val.length()) ) ) );
+		return QVariant( QRect( pointFromString( val.mid(pivot+2,val.length()) ), pointFromString( val.mid(0,pivot+1) ) ) );
 	    return QVariant( QRect() );
 	}
     case QVariant::PointArray: // format '((x,y),(x1,y1),...,(xn,yn))'
@@ -301,7 +344,7 @@ QVariant QPSQLResult::data( int i )
 		int start = val.find( pointPattern, idx );
 		int end = -1;
 		if ( start != -1 ) {
-		    end = val.find( QRegExp("\\)"), start+1 );
+		    end = val.find( ')', start+1 );
 		    if ( end != -1 ) {
 			parray.setPoint( i, pointFromString( val.mid(idx, end-idx+1) ) );
 		    }
@@ -316,6 +359,29 @@ QVariant QPSQLResult::data( int i )
 	    return QVariant( parray );
 	}
     case QVariant::ByteArray: {
+	if ( ptype == BYTEAOID ) {
+	    uint i = 0;
+	    int index = 0;
+	    uint len = val.length();
+	    static const QChar backslash( '\\' );
+	    QByteArray ba( (int)len );
+	    while ( i < len ) {
+		if ( val.at( i ) == backslash ) {
+		    if ( val.at( i + 1 ).isDigit() ) {
+			ba[ index++ ] = (char)(val.mid( i + 1, 3 ).toInt( 0, 8 ));
+			i += 4;
+		    } else {
+			ba[ index++ ] = val.at( i + 1 );
+			i += 2;
+		    }
+		} else {
+		    ba[ index++ ] = val.at( i++ ).unicode();
+		}
+	    }
+	    ba.resize( index );
+	    return QVariant( ba );
+	}
+
 	QByteArray ba;
 	((QSqlDriver*)driver())->beginTransaction();
 	Oid oid = val.toInt();
@@ -386,7 +452,7 @@ bool QPSQLResult::reset ( const QString& query )
     cleanup();
     if ( !driver() )
 	return FALSE;
-    if ( !driver()-> isOpen() || driver()->isOpenError() )
+    if ( !driver()->isOpen() || driver()->isOpenError() )
 	return FALSE;
     setActive( FALSE );
     setAt( QSql::BeforeFirst );
@@ -399,8 +465,13 @@ bool QPSQLResult::reset ( const QString& query )
     }
     int status =  PQresultStatus( d->result );
     if ( status == PGRES_COMMAND_OK || status == PGRES_TUPLES_OK ) {
-	setSelect( (status == PGRES_TUPLES_OK) );
-	currentSize = PQntuples( d->result );
+	if ( status == PGRES_TUPLES_OK ) {
+	    setSelect( TRUE );
+	    currentSize = PQntuples( d->result );
+	} else {
+	    setSelect( FALSE );
+	    currentSize = -1;
+	}
 	setActive( TRUE );
 	return TRUE;
     }
@@ -415,19 +486,90 @@ int QPSQLResult::size()
 
 int QPSQLResult::numRowsAffected()
 {
-    return isSelect() ? size() : QString( PQcmdTuples( d->result ) ).toInt();
+    return QString( PQcmdTuples( d->result ) ).toInt();
 }
 
 ///////////////////////////////////////////////////////////////////
 
+static bool setEncodingUtf8( PGconn* connection )
+{
+    PGresult* result = PQexec( connection, "SET CLIENT_ENCODING TO 'UNICODE'" );
+    int status = PQresultStatus( result );
+    PQclear( result );
+    return status == PGRES_COMMAND_OK;
+}
+
+static void setDatestyle( PGconn* connection )
+{
+    PGresult* result = PQexec( connection, "SET DATESTYLE TO 'ISO'" );
+#ifdef QT_CHECK_RANGE
+    int status =  PQresultStatus( result );
+    if ( status != PGRES_COMMAND_OK )
+        qWarning( "%s", PQerrorMessage( connection ) );
+#endif
+    PQclear( result );
+}
+
+static QPSQLDriver::Protocol getPSQLVersion( PGconn* connection )
+{
+    PGresult* result = PQexec( connection, "select version()" );
+    int status =  PQresultStatus( result );
+    if ( status == PGRES_COMMAND_OK || status == PGRES_TUPLES_OK ) {
+	QString val( PQgetvalue( result, 0, 0 ) );
+	PQclear( result );
+	QRegExp rx( "(\\d+)\\.(\\d+)" );
+	rx.setMinimal ( TRUE ); // enforce non-greedy RegExp
+        if ( rx.search( val ) != -1 ) {
+	    int vMaj = rx.cap( 1 ).toInt();
+	    int vMin = rx.cap( 2 ).toInt();
+	    if ( vMaj < 6 ) {
+#ifdef QT_CHECK_RANGE
+		qWarning( "This version of PostgreSQL is not supported and may not work." );
+#endif
+		return QPSQLDriver::Version6;
+	    }
+	    if ( vMaj == 6 ) {
+		return QPSQLDriver::Version6;
+	    } else if ( vMaj == 7 ) {
+		if ( vMin < 1 )
+		    return QPSQLDriver::Version7;
+		else if ( vMin < 3 )
+		    return QPSQLDriver::Version71;
+	    }
+	    return QPSQLDriver::Version73;
+	}
+    } else {
+#ifdef QT_CHECK_RANGE
+	qWarning( "This version of PostgreSQL is not supported and may not work." );
+#endif
+    }
+
+    return QPSQLDriver::Version6;
+}
+
 QPSQLDriver::QPSQLDriver( QObject * parent, const char * name )
     : QSqlDriver(parent,name ? name : "QPSQL"), pro( QPSQLDriver::Version6 )
 {
     init();
 }
 
+QPSQLDriver::QPSQLDriver( PGconn * conn, QObject * parent, const char * name )
+    : QSqlDriver(parent,name ? name : "QPSQL"), pro( QPSQLDriver::Version6 )
+{
+    init();
+    d->connection = conn;
+    if ( conn ) {
+	pro = getPSQLVersion( d->connection );
+	setOpen( TRUE );
+	setOpenError( FALSE );
+    }
+}
+
 void QPSQLDriver::init()
 {
+    qSqlDriverExtDict()->insert( this, new QPSQLDriverExtension(this) );
+    qSqlOpenExtDict()->insert( this, new QPSQLOpenExtension(this) );
+
     d = new QPSQLPrivate();
 }
 
@@ -436,6 +578,14 @@ QPSQLDriver::~QPSQLDriver()
     if ( d->connection )
 	PQfinish( d->connection );
     delete d;
+    if ( !qSqlDriverExtDict()->isEmpty() ) {
+	QSqlDriverExtension *ext = qSqlDriverExtDict()->take( this );
+	delete ext;
+    }
+    if ( !qSqlOpenExtDict()->isEmpty() ) {
+	QSqlOpenExtension *ext = qSqlOpenExtDict()->take( this );
+	delete ext;
+    }
 }
 
 PGconn* QPSQLDriver::connection()
@@ -452,73 +602,49 @@ bool QPSQLDriver::hasFeature( DriverFeature f ) const
     case QuerySize:
 	return TRUE;
     case BLOB:
-	return FALSE;
+	return pro >= QPSQLDriver::Version71;
+    case Unicode:
+	return d->isUtf8;
     default:
 	return FALSE;
     }
 }
 
-static QPSQLDriver::Protocol getPSQLVersion( PGconn* connection )
+bool QPSQLDriver::open( const QString&,
+			const QString&,
+			const QString&,
+			const QString&,
+			int )
 {
-    PGresult* result = PQexec( connection, "select version()" );
-    int status =  PQresultStatus( result );
-    if ( status == PGRES_COMMAND_OK || status == PGRES_TUPLES_OK ) {
-	QString val( PQgetvalue( result, 0, 0 ) );
-	PQclear( result );
-	QRegExp rx( "(\\d*)\\.(\\d*)" );
-	rx.setMinimal ( TRUE ); // enforce non-greedy RegExp
-        if ( rx.search( val ) != -1 ) {
-	    int vMaj = rx.cap( 1 ).toInt();
-	    int vMin = rx.cap( 2 ).toInt();
-	    if ( vMaj < 6 ) {
-#ifdef QT_CHECK_RANGE
-		qWarning( "This version of PostgreSQL is not supported and may not work." );
-#endif
-		return QPSQLDriver::Version6;
-	    }
-	    if ( vMaj == 6 ) {
-		return QPSQLDriver::Version6;
-	    }
-	    if ( vMaj == 7 ) {
-		if ( vMin < 1 ) {
-		    return QPSQLDriver::Version7;
-		} else {
-		    return QPSQLDriver::Version71;
-		}
-	    }
-	    if ( vMaj > 7 ) {
-		return QPSQLDriver::Version71;
-	    }
-	}
-    } else {
-#ifdef QT_CHECK_RANGE
-	qWarning( "This version of PostgreSQL is not supported and may not work." );
-#endif
-    }
-
-    return QPSQLDriver::Version6;
+    qWarning("QPSQLDriver::open(): This version of open() is no longer supported." );
+    return FALSE;
 }
 
 bool QPSQLDriver::open( const QString & db,
 			const QString & user,
 			const QString & password,
 			const QString & host,
-			int port )
+			int port,
+			const QString& connOpts )
 {
-    int status;
     if ( isOpen() )
 	close();
     QString connectString;
     if ( host.length() )
-	connectString += QString("host=%1 ").arg( host );
+	connectString.append( "host=" ).append( host );
     if ( db.length() )
-	connectString += QString("dbname=%1 ").arg( db );
+	connectString.append( " dbname=" ).append( db );
     if ( user.length() )
-	connectString += QString("user=%1 ").arg( user );
+	connectString.append( " user=" ).append( user );
     if ( password.length() )
-	connectString += QString("password=%1 ").arg( password );
+	connectString.append( " password=" ).append( password );
     if ( port > -1 )
-	connectString += QString("port=%1 ").arg( port );
+	connectString.append( " port=" ).append( QString::number( port ) );
+
+    // add any connect options - the server will handle error detection
+    if ( !connOpts.isEmpty() )
+	connectString += " " + QStringList::split( ';', connOpts ).join( " " );
+
     d->connection = PQconnectdb( connectString.local8Bit().data() );
     if ( PQstatus( d->connection ) == CONNECTION_BAD ) {
 	setLastError( qMakeError("Unable to connect", QSqlError::Connection, d ) );
@@ -526,41 +652,20 @@ bool QPSQLDriver::open( const QString & db,
 	return FALSE;
     }
 
-// Unicode support is only working if the client library has been compiled with
-// multibyte support.
-#ifdef MULTIBYTE
-    status = PQsetClientEncoding( d->connection, "UNICODE" );
-    if ( status == 0 ) {
-	d->isUtf8 = TRUE;
-    }
-#endif
-
     pro = getPSQLVersion( d->connection );
+    d->isUtf8 = setEncodingUtf8( d->connection );
+    setDatestyle( d->connection );
 
-    PGresult* dateResult = 0;
-    switch( pro ) {
-    case QPSQLDriver::Version6:
-	dateResult = PQexec( d->connection, "SET DATESTYLE TO 'ISO'" );
-	break;
-    case QPSQLDriver::Version7:
-    case QPSQLDriver::Version71:
-	dateResult = PQexec( d->connection, "SET DATESTYLE=ISO" );
-	break;
-    }
-#ifdef QT_CHECK_RANGE
-    status =  PQresultStatus( dateResult );
-    if ( status != PGRES_COMMAND_OK )
-	qWarning( PQerrorMessage( d->connection ) );
-#endif
     setOpen( TRUE );
-
+    setOpenError( FALSE );
     return TRUE;
 }
 
 void QPSQLDriver::close()
 {
     if ( isOpen() ) {
-	PQfinish( d->connection );
+        if (d->connection)
+	    PQfinish( d->connection );
 	d->connection = 0;
 	setOpen( FALSE );
 	setOpenError( FALSE );
@@ -626,19 +731,50 @@ bool QPSQLDriver::rollbackTransaction()
     return TRUE;
 }
 
-QStringList QPSQLDriver::tables( const QString& /* user */ ) const
+QStringList QPSQLDriver::tables( const QString& typeName ) const
 {
     QStringList tl;
     if ( !isOpen() )
 	return tl;
+    int type = typeName.toInt();
     QSqlQuery t = createQuery();
-    QString stmt;
-    stmt = "select relname from pg_class where ( relkind = 'r' or relkind = 'v' ) "
-		"and ( relname !~ '^Inv' ) "
-		"and ( relname !~ '^pg_' ) ";
-    t.exec( stmt );
-    while ( t.isActive() && t.next() )
-	tl.append( t.value(0).toString() );
+    t.setForwardOnly( TRUE );
+
+    if ( typeName.isEmpty() || ((type & (int)QSql::Tables) == (int)QSql::Tables) ) {
+
+        QString query("select relname from pg_class where (relkind = 'r') "
+                      "and (relname !~ '^Inv') "
+                      "and (relname !~ '^pg_') ");
+        if (pro >= QPSQLDriver::Version73)
+            query.append("and (relnamespace not in "
+                         "(select oid from pg_namespace where nspname = 'information_schema')) "
+                         "and pg_table_is_visible(pg_class.oid) ");
+        t.exec(query);
+	while ( t.next() )
+	    tl.append( t.value(0).toString() );
+    }
+    if ( (type & (int)QSql::Views) == (int)QSql::Views ) {
+        QString query("select relname from pg_class where ( relkind = 'v' ) "
+                "and ( relname !~ '^Inv' ) "
+                "and ( relname !~ '^pg_' ) ");
+        if (pro >= QPSQLDriver::Version73)
+            query.append("and (relnamespace not in "
+                         "(select oid from pg_namespace where nspname = 'information_schema')) "
+                         "and pg_table_is_visible(pg_class.oid) ");
+        t.exec(query);
+	while ( t.next() )
+	    tl.append( t.value(0).toString() );
+    }
+    if ( (type & (int)QSql::SystemTables) == (int)QSql::SystemTables ) {
+        QString query( "select relname from pg_class where ( relkind = 'r' ) "
+		"and ( relname like 'pg_%' ) " );
+        if (pro >= QPSQLDriver::Version73)
+		query.append( "and pg_table_is_visible(pg_class.oid) " );
+        t.exec(query);
+	while ( t.next() )
+	    tl.append( t.value(0).toString() );
+    }
+
     return tl;
 }
 
@@ -654,24 +790,29 @@ QSqlIndex QPSQLDriver::primaryIndex( const QString& tablename ) const
     case QPSQLDriver::Version6:
 	stmt = "select pg_att1.attname, int(pg_att1.atttypid), pg_att2.attnum, pg_cl.relname "
 		"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
-		"where pg_cl.relname = '%1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
-		"and pg_att2.attrelid = pg_ind.indexrelid "
-		"and pg_att1.attrelid = pg_ind.indrelid "
-		"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
-		"order by pg_att2.attnum";
+		"where lower(pg_cl.relname) = '%1_pkey' ";
 	break;
     case QPSQLDriver::Version7:
     case QPSQLDriver::Version71:
 	stmt = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
 		"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
-		"where pg_cl.relname = '%1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
-		"and pg_att2.attrelid = pg_ind.indexrelid "
-		"and pg_att1.attrelid = pg_ind.indrelid "
-		"and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
-		"order by pg_att2.attnum";
+		"where lower(pg_cl.relname) = '%1_pkey' ";
+	break;
+    case QPSQLDriver::Version73:
+	stmt = "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
+		"from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
+		"where lower(pg_cl.relname) = '%1_pkey' "
+		"and pg_table_is_visible(pg_cl.oid) "
+		"and pg_att1.attisdropped = false ";
 	break;
     }
-    i.exec( stmt.arg( tablename ) );
+    stmt += "and pg_cl.oid = pg_ind.indexrelid "
+	    "and pg_att2.attrelid = pg_ind.indexrelid "
+	    "and pg_att1.attrelid = pg_ind.indrelid "
+	    "and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
+	    "order by pg_att2.attnum";
+
+    i.exec( stmt.arg( tablename.lower() ) );
     while ( i.isActive() && i.next() ) {
 	QSqlField f( i.value(0).toString(), qDecodePSQLType( i.value(1).toInt() ) );
 	idx.append( f );
@@ -690,7 +831,7 @@ QSqlRecord QPSQLDriver::record( const QString& tablename ) const
     case QPSQLDriver::Version6:
 	stmt = "select pg_attribute.attname, int(pg_attribute.atttypid) "
 			"from pg_class, pg_attribute "
-			"where pg_class.relname = '%1' "
+			"where lower(pg_class.relname) = '%1' "
 			"and pg_attribute.attnum > 0 "
 			"and pg_attribute.attrelid = pg_class.oid ";
 	break;
@@ -698,14 +839,23 @@ QSqlRecord QPSQLDriver::record( const QString& tablename ) const
     case QPSQLDriver::Version71:
 	stmt = "select pg_attribute.attname, pg_attribute.atttypid::int "
 			"from pg_class, pg_attribute "
-			"where pg_class.relname = '%1' "
+			"where lower(pg_class.relname) = '%1' "
 			"and pg_attribute.attnum > 0 "
 			"and pg_attribute.attrelid = pg_class.oid ";
 	break;
+    case QPSQLDriver::Version73:
+	stmt = "select pg_attribute.attname, pg_attribute.atttypid::int "
+			"from pg_class, pg_attribute "
+			"where lower(pg_class.relname) = '%1' "
+			"and pg_table_is_visible(pg_class.oid) "
+			"and pg_attribute.attnum > 0 "
+			"and pg_attribute.attisdropped = false "
+			"and pg_attribute.attrelid = pg_class.oid ";
+	break;
     }
 
     QSqlQuery fi = createQuery();
-    fi.exec( stmt.arg( tablename ) );
+    fi.exec( stmt.arg( tablename.lower() ) );
     while ( fi.next() ) {
 	QSqlField f( fi.value(0).toString(), qDecodePSQLType( fi.value(1).toInt() ) );
 	fil.append( f );
@@ -743,7 +893,7 @@ QSqlRecordInfo QPSQLDriver::recordInfo( const QString& tablename ) const
 	stmt = "select pg_attribute.attname, int(pg_attribute.atttypid), pg_attribute.attnotnull, "
 		"pg_attribute.attlen, pg_attribute.atttypmod, int(pg_attribute.attrelid), pg_attribute.attnum "
 		"from pg_class, pg_attribute "
-		"where pg_class.relname = '%1' "
+		"where lower(pg_class.relname) = '%1' "
 		"and pg_attribute.attnum > 0 "
 		"and pg_attribute.attrelid = pg_class.oid ";
 	break;
@@ -751,7 +901,7 @@ QSqlRecordInfo QPSQLDriver::recordInfo( const QString& tablename ) const
 	stmt = "select pg_attribute.attname, pg_attribute.atttypid::int, pg_attribute.attnotnull, "
 		"pg_attribute.attlen, pg_attribute.atttypmod, pg_attribute.attrelid::int, pg_attribute.attnum "
 		"from pg_class, pg_attribute "
-		"where pg_class.relname = '%1' "
+		"where lower(pg_class.relname) = '%1' "
 		"and pg_attribute.attnum > 0 "
 		"and pg_attribute.attrelid = pg_class.oid ";
 	break;
@@ -760,15 +910,28 @@ QSqlRecordInfo QPSQLDriver::recordInfo( const QString& tablename ) const
 		"pg_attribute.attlen, pg_attribute.atttypmod, pg_attrdef.adsrc "
 		"from pg_class, pg_attribute "
 		"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
-		"where pg_class.relname = '%1' "
+		"where lower(pg_class.relname) = '%1' "
 		"and pg_attribute.attnum > 0 "
-		"and pg_attribute.attrelid = pg_class.oid ";
+		"and pg_attribute.attrelid = pg_class.oid "
+		"order by pg_attribute.attnum ";
+	break;
+    case QPSQLDriver::Version73:
+	stmt = "select pg_attribute.attname, pg_attribute.atttypid::int, pg_attribute.attnotnull, "
+		"pg_attribute.attlen, pg_attribute.atttypmod, pg_attrdef.adsrc "
+		"from pg_class, pg_attribute "
+		"left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
+		"where lower(pg_class.relname) = '%1' "
+		"and pg_table_is_visible(pg_class.oid) "
+		"and pg_attribute.attnum > 0 "
+		"and pg_attribute.attrelid = pg_class.oid "
+		"and pg_attribute.attisdropped = false "
+		"order by pg_attribute.attnum ";
 	break;
     }
 
     QSqlQuery query = createQuery();
-    query.exec( stmt.arg( tablename ) );
-    if ( pro == QPSQLDriver::Version71 ) {
+    query.exec( stmt.arg( tablename.lower() ) );
+    if ( pro >= QPSQLDriver::Version71 ) {
 	while ( query.next() ) {
 	    int len = query.value( 3 ).toInt();
 	    int precision = query.value( 4 ).toInt();
@@ -792,8 +955,8 @@ QSqlRecordInfo QPSQLDriver::recordInfo( const QString& tablename ) const
 	// Postgres < 7.1 cannot handle outer joins
 	while ( query.next() ) {
 	    QString defVal;
-	    QString stmt2 = ( "select pg_attrdef.adsrc from pg_attrdef where "
-				"pg_attrdef.adrelid = %1 and pg_attrdef.adnum = %2 " );
+	    QString stmt2 = "select pg_attrdef.adsrc from pg_attrdef where "
+			    "pg_attrdef.adrelid = %1 and pg_attrdef.adnum = %2 ";
 	    QSqlQuery query2 = createQuery();
 	    query2.exec( stmt2.arg( query.value( 5 ).toInt() ).arg( query.value( 6 ).toInt() ) );
 	    if ( query2.isActive() && query2.next() )
@@ -861,20 +1024,58 @@ QString QPSQLDriver::formatValue( const QSqlField* field,
 	    if ( field->value().toDateTime().isValid() ) {
 		QDate dt = field->value().toDateTime().date();
 		QTime tm = field->value().toDateTime().time();
-		r = "'" + QString::number(dt.year()) + "-" +
-			  QString::number(dt.month()) + "-" +
-			  QString::number(dt.day()) + " " +
+		// msecs need to be right aligned otherwise psql
+		// interpretes them wrong
+		r = "'" + QString::number( dt.year() ) + "-" +
+			  QString::number( dt.month() ) + "-" +
+			  QString::number( dt.day() ) + " " +
 			  tm.toString() + "." +
-			  QString::number(tm.msec()) + "'";
+			  QString::number( tm.msec() ).rightJustify( 3, '0' ) + "'";
 	    } else {
 		r = nullText();
 	    }
 	    break;
+	case QVariant::Time:
+	    if ( field->value().toTime().isValid() ) {
+		r = field->value().toTime().toString( Qt::ISODate );
+	    } else {
+		r = nullText();
+	    }
 	case QVariant::String:
 	case QVariant::CString: {
-	    // Escape '\' characters
-	    r = QSqlDriver::formatValue( field );
-	    r.replace( QRegExp( "\\\\" ), "\\\\" );
+	    switch ( field->value().type() ) {
+		case QVariant::Rect: {
+		    QRect rec = field->value().toRect();
+		    // upper right corner then lower left according to psql docs
+		    r = "'(" + QString::number( rec.right() ) +
+			"," + QString::number( rec.bottom() ) +
+			"),(" + QString::number( rec.left() ) +
+			"," + QString::number( rec.top() ) + ")'";
+		    break;
+		}
+		case QVariant::Point: {
+		    QPoint p = field->value().toPoint();
+		    r = "'(" + QString::number( p.x() ) +
+			"," + QString::number( p.y() ) + ")'";
+		    break;
+		}
+		case QVariant::PointArray: {
+		    QPointArray pa = field->value().toPointArray();
+		    r = "' ";
+		    for ( int i = 0; i < (int)pa.size(); ++i ) {
+			r += "(" + QString::number( pa[i].x() ) +
+			     "," + QString::number( pa[i].y() ) + "),";
+		    }
+		    r.truncate( r.length() - 1 );
+		    r += "'";
+		    break;
+		}
+		default:
+		    // Escape '\' characters
+		    r = QSqlDriver::formatValue( field );
+		    r.replace( "\\", "\\\\" );
+		    break;
+	    }
 	    break;
 	}
 	case QVariant::Bool:
@@ -883,12 +1084,23 @@ QString QPSQLDriver::formatValue( const QSqlField* field,
 	    else
 		r = "FALSE";
 	    break;
-	case QVariant::ByteArray:
-#ifdef QT_CHECKRANGE
-	    // bytearrays cannot be inserted directly into postgresql
-	    qWarning( "QPSQLDriver::formatValue: cannot format ByteArray." );
-#endif
+	case QVariant::ByteArray: {
+	    QByteArray ba = field->value().asByteArray();
+	    QString res;
+	    r = "'";
+	    unsigned char uc;
+	    for ( int i = 0; i < (int)ba.size(); ++i ) {
+		uc = (unsigned char) ba[ i ];
+		if ( uc > 40 && uc < 92 ) {
+		    r += uc;
+		} else {
+		    r += "\\\\";
+		    r += QString::number( (unsigned char) ba[ i ], 8 ).rightJustify( 3, '0', TRUE );
+		}
+	    }
+	    r += "'";
 	    break;
+	}
 	default:
 	    r = QSqlDriver::formatValue( field );
 	    break;
