@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qglobal.h   3.2.3   edited Nov 10 11:21 $
+** $Id: qt/qglobal.h   3.3.6   edited Mar 8 17:47 $
 **
 ** Global type declarations and definitions
 **
 ** Created : 920529
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -38,11 +38,11 @@
 #ifndef QGLOBAL_H
 #define QGLOBAL_H
 
-#define QT_VERSION_STR   "3.2.3"
+#define QT_VERSION_STR   "3.3.6"
 /*
    QT_VERSION is (major << 16) + (minor << 8) + patch.
  */
-#define QT_VERSION 0x030203
+#define QT_VERSION 0x030306
 
 /*
    The operating system, must be one of: (Q_OS_x)
@@ -81,7 +81,7 @@
 
 #if defined(__DARWIN_X11__)
 #  define Q_OS_DARWIN
-#elif defined(__APPLE__) && defined(__GNUC__)
+#elif defined(__APPLE__) && (defined(__GNUC__) || defined(__xlC__))
 #  define Q_OS_MACX
 #elif defined(__MACOSX__)
 #  define Q_OS_MACX
@@ -114,7 +114,7 @@
 #  define Q_OS_RELIANT
 #elif defined(__linux__) || defined(__linux)
 #  define Q_OS_LINUX
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
 #  define Q_OS_FREEBSD
 #  define Q_OS_BSD4
 #elif defined(__NetBSD__)
@@ -171,6 +171,23 @@
 #  define Q_OS_UNIX
 #endif
 
+#if defined(Q_OS_MACX)
+#  ifdef MAC_OS_X_VERSION_MIN_REQUIRED
+#    undef MAC_OS_X_VERSION_MIN_REQUIRED
+#  endif
+#  define MAC_OS_X_VERSION_MIN_REQUIRED MAC_OS_X_VERSION_10_2
+#  include <AvailabilityMacros.h>
+#  if !defined(MAC_OS_X_VERSION_10_3)
+#     define MAC_OS_X_VERSION_10_3 MAC_OS_X_VERSION_10_2 + 1
+#  endif
+#  if !defined(MAC_OS_X_VERSION_10_4)
+#       define MAC_OS_X_VERSION_10_4 MAC_OS_X_VERSION_10_3 + 1
+#  endif
+#  if (MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_4)
+#    error "This version of Mac OS X is unsupported"
+#  endif
+#endif
+
 
 /*
    The compiler, must be one of: (Q_CC_x)
@@ -178,7 +195,7 @@
      SYM	- Symantec C++ for both PC and Macintosh
      MPW	- MPW C++
      MWERKS	- Metrowerks CodeWarrior
-     MSVC	- Microsoft Visual C/C++
+     MSVC	- Microsoft Visual C/C++, Intel C++ for Windows
      BOR	- Borland/Turbo C++
      WAT	- Watcom C++
      GNU	- GNU C++
@@ -193,7 +210,7 @@
      USLC	- SCO OUDK, UDK, and UnixWare 2.X C++
      CDS	- Reliant C++
      KAI	- KAI C++
-     INTEL	- Intel C++
+     INTEL	- Intel C++ for Linux, Intel C++ for Windows
      HIGHC	- MetaWare High C/C++
      PGI	- Portland Group C++
      GHS	- Green Hills Optimizing C++ Compilers
@@ -223,7 +240,6 @@
 #  define Q_CC_MWERKS
 /* "explicit" recognized since 4.0d1 */
 #  define QMAC_PASCAL pascal
-#  define Q_NO_USING_KEYWORD /* ### check "using" status */
 
 #elif defined(_MSC_VER)
 #  define Q_CC_MSVC
@@ -233,11 +249,19 @@
 /* Visual C++.Net issues for _MSC_VER >= 1300 */
 #  if _MSC_VER >= 1300
 #    define Q_CC_MSVC_NET
-#    if _MSC_VER < 1310
+#    if _MSC_VER < 1310 || (defined(Q_OS_WIN64) && defined(_M_IA64))
 #      define Q_TYPENAME
 #    endif
 #  endif
-#  define Q_NO_USING_KEYWORD /* ### check "using" status */
+/* Intel C++ disguising as Visual C++: the `using' keyword avoids warnings */
+#  if defined(__INTEL_COMPILER)
+#    define Q_CC_INTEL
+#    if !defined(__EXCEPTIONS)
+#      define Q_NO_EXCEPTIONS
+#    endif
+#  else
+#    define Q_NO_USING_KEYWORD /* ### check "using" status */
+#  endif
 
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
 #  define Q_CC_BOR
@@ -285,8 +309,17 @@
 #  if defined(Q_OS_HPUX) && __GNUC__ == 3 && __GNUC_MINOR__ >= 1
 #    define Q_WRONG_SB_CTYPE_MACROS
 #  endif
+
+/* ARM gcc pads structs to 32 bits, even when they contain a single
+   char, or short.  We tell gcc to pack QChars to 16 bits, to avoid
+   QString bloat. However, gcc 3.4 doesn't allow us to create references to
+   members of a packed struct. (Pointers are OK, because then you
+   supposedly know what you are doing.) */
 #  if (defined(__arm__) || defined(__ARMEL__)) && !defined(QT_MOC_CPP)
 #    define Q_PACKED __attribute__ ((packed))
+#    if __GNUC__ == 3 && __GNUC_MINOR__ >= 4
+#      define Q_NO_PACKED_REFERENCE
+#    endif
 #  endif
 #  if !defined(__EXCEPTIONS)
 #    define Q_NO_EXCEPTIONS
@@ -332,7 +365,7 @@
    Compaq C++ V6.3-002.
    This compiler is different enough from other EDG compilers to handle
    it separately anyway. */
-#elif defined(__DECCXX)
+#elif defined(__DECCXX) || defined(__DECC)
 #  define Q_CC_DEC
 /* Compaq C++ V6 compilers are EDG-based but I'm not sure about older
    DEC C++ V5 compilers. */
@@ -373,7 +406,7 @@
 #    define Q_NO_BOOL_TYPE
 #  endif
 
-/* The Portland Group compiler is based on EDG and does define __EDG__ */
+/* The Comeau compiler is based on EDG and does define __EDG__ */
 #  if defined(__COMO__)
 #    define Q_CC_COMEAU
 #    define Q_C_CALLBACKS
@@ -389,7 +422,7 @@
 #    endif
 #    define Q_NO_USING_KEYWORD
 
-/* Using the `using' keyword avoids Intel C++ warnings */
+/* Using the `using' keyword avoids Intel C++ for Linux warnings */
 #  elif defined(__INTEL_COMPILER)
 #    define Q_CC_INTEL
 #    if !defined(__EXCEPTIONS)
@@ -434,7 +467,6 @@
    such as template specialization nevertheless */
 #  elif defined(__sgi)
 #    define Q_CC_MIPS
-#    define Q_STRICT_INLINING_RULES
 #    if defined(_MIPS_SIM) && (_MIPS_SIM == _ABIO32) /* o32 ABI */
 #      define Q_TYPENAME
 #      define Q_BROKEN_TEMPLATE_SPECIALIZATION
@@ -445,6 +477,9 @@
 #      define Q_BROKEN_TEMPLATE_SPECIALIZATION
 #    endif
 #    define Q_NO_USING_KEYWORD /* ### check "using" status */
+#    if defined(_COMPILER_VERSION) && (_COMPILER_VERSION >= 740)
+#      pragma set woff 3624,3625, 3649 /* turn off some harmless warnings */
+#    endif
 #  endif
 
 /* The older UnixWare 2.X compiler? */
@@ -460,7 +495,7 @@
 #elif defined(__HIGHC__)
 #  define Q_CC_HIGHC
 
-#elif defined(__SUNPRO_CC)
+#elif defined(__SUNPRO_CC) || defined(__SUNPRO_C)
 #  define Q_CC_SUN
 /* 5.0 compiler or better
     'bool' is enabled by default but can be disabled using -features=nobool
@@ -804,7 +839,7 @@ class QDataStream;
 
 #ifndef QT_H
 #include "qfeatures.h"
-#endif // QT_H
+#endif /* QT_H */
 
 
 //
@@ -906,7 +941,6 @@ Q_EXPORT int qWinVersion();
 //
 // Use to avoid "unused parameter" warnings
 //
-
 #define Q_UNUSED(x) (void)x;
 
 //
@@ -1041,14 +1075,16 @@ Q_EXPORT const char *qInstallPathData();
 Q_EXPORT const char *qInstallPathTranslations();
 Q_EXPORT const char *qInstallPathSysconf();
 
-#endif // __cplusplus
+#endif /* __cplusplus */
 
-// compilers which follow outdated template instantiation rules
-// require a class to have a comparison operator to exist when
-// a QValueList of this type is instantiated. It's not actually
-// used in the list, though. Hence the dummy implementation.
-// Just in case other code relies on it we better trigger a warning
-// mandating a real implementation.
+/*
+ compilers which follow outdated template instantiation rules
+ require a class to have a comparison operator to exist when
+ a QValueList of this type is instantiated. It's not actually
+ used in the list, though. Hence the dummy implementation.
+ Just in case other code relies on it we better trigger a warning
+ mandating a real implementation.
+*/
 #ifdef Q_FULL_TEMPLATE_INSTANTIATION
 #  define Q_DUMMY_COMPARISON_OPERATOR(C) \
     bool operator==( const C& ) const { \
@@ -1059,13 +1095,13 @@ Q_EXPORT const char *qInstallPathSysconf();
 #  define Q_DUMMY_COMPARISON_OPERATOR(C)
 #endif
 
-#endif // QGLOBAL_H
+#endif /* QGLOBAL_H */
 
-//
-// Avoid some particularly useless warnings from some stupid compilers.
-// To get ALL C++ compiler warnings, define QT_CC_WARNINGS or comment out
-// the line "#define QT_NO_WARNINGS"
-//
+/*
+ Avoid some particularly useless warnings from some stupid compilers.
+ To get ALL C++ compiler warnings, define QT_CC_WARNINGS or comment out
+ the line "#define QT_NO_WARNINGS"
+*/
 
 #if !defined(QT_CC_WARNINGS)
 #  define QT_NO_WARNINGS
@@ -1093,3 +1129,4 @@ Q_EXPORT const char *qInstallPathSysconf();
 #    pragma warn -sig
 #  endif
 #endif
+
