@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qgvector.cpp   3.3.4   edited May 27 2003 $
+** $Id: qt/qgvector.cpp   3.3.6   edited Aug 31 2005 $
 **
 ** Implementation of QGVector class
 **
 ** Created : 930907
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -324,19 +324,23 @@ bool QGVector::resize( uint newsize )		// resize array
 	    return FALSE;
 	}
 	Item *newvec = NEW(Item,newsize);		// manual realloc
+        if (!newvec)
+            return FALSE;
 	memcpy( newvec, vec, (len < newsize ? len : newsize)*sizeof(Item) );
 	DELETE(vec);
 	vec = newvec;
 #else
-	vec = (Item*)realloc( (char *)vec, newsize*sizeof(Item) );
+	Item *newvec = (Item*)realloc( (char *)vec, newsize*sizeof(Item) );
+        if (!newvec)
+            return FALSE;
+        vec = newvec;
 #endif
     } else {					// create new vector
 	vec = NEW(Item,newsize);
+        if (!vec)
+            return FALSE;
 	len = numItems = 0;
     }
-    Q_CHECK_PTR( vec );
-    if ( !vec )					// no memory
-	return FALSE;
     if ( newsize > len )			// init extra space added
 	memset( (void*)&vec[len], 0, (newsize-len)*sizeof(Item) );
     len = newsize;
