@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id: qt/qlineedit.cpp   3.2.3   edited Oct 20 08:04 $
+** $Id: qt/qlineedit.cpp   3.3.6   edited Feb 15 10:13 $
 **
 ** Implementation of QLineEdit widget class
 **
 ** Created : 941011
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -260,7 +260,7 @@ struct QLineEditPrivate : public Qt
     // drag and drop
     QPoint dndPos;
     int dndTimer;
-    void drag();
+    bool drag();
 #endif
 };
 
@@ -437,6 +437,7 @@ QLineEdit::QLineEdit( const QString& contents, const QString &inputMask, QWidget
 
 QLineEdit::~QLineEdit()
 {
+    delete [] d->maskData;
     delete d;
 }
 
@@ -456,9 +457,10 @@ QLineEdit::~QLineEdit()
 */
 QString QLineEdit::text() const
 {
+    QString res = d->text;
     if ( d->maskData )
-	return d->stripString( d->text );
-    return ( d->text.isNull() ? QString::fromLatin1("") : d->text );
+	res = d->stripString( d->text );
+    return ( res.isNull() ? QString::fromLatin1("") : res );
 }
 
 void QLineEdit::setText( const QString& text)
@@ -582,6 +584,8 @@ QLineEdit::EchoMode QLineEdit::echoMode() const
 
 void QLineEdit::setEchoMode( EchoMode mode )
 {
+    if (mode == (EchoMode)d->echoMode)
+	return;
     d->echoMode = mode;
     d->updateTextLayout();
     update();
@@ -678,6 +682,9 @@ int QLineEdit::cursorPosition() const
 
 void QLineEdit::setCursorPosition( int pos )
 {
+    if (pos < 0)
+        pos = 0;
+
     if ( pos <= (int) d->text.length() )
 	d->moveCursor( pos );
 }
@@ -995,7 +1002,7 @@ bool QLineEdit::getSelection( int *start, int *end )
 /*!
     Selects text from position \a start and for \a length characters.
 
-    \sa deselect() selectAll() getSelection()
+    \sa deselect() selectAll() getSelection() cursorForward() cursorBackward()
 */
 
 void QLineEdit::setSelection( int start, int length )
@@ -1312,7 +1319,6 @@ void QLineEdit::copy() const
 
 void QLineEdit::paste()
 {
-    d->removeSelectedText();
     insert( QApplication::clipboard()->text( QClipboard::Clipboard ) );
 }
 
@@ -1347,8 +1353,7 @@ bool QLineEdit::event( QEvent * e )
 	     || ke->state() == Keypad ) {
 	    if ( ke->key() < Key_Escape ) {
 		ke->accept();
-	    } else if ( ke->state() == NoButton
-			|| ke->state() == ShiftButton ) {
+	    } else {
 		switch ( ke->key() ) {
   		case Key_Delete:
   		case Key_Home:
@@ -1396,7 +1401,8 @@ bool QLineEdit::event( QEvent * e )
 		d->setCursorVisible( !d->cursorVisible );
 #ifndef QT_NO_DRAGANDDROP
 	} else if ( timerId == d->dndTimer ) {
-	    d->drag();
+	    if( !d->drag() )
+                return TRUE;
 #endif
 	} else if ( timerId == d->tripleClickTimer ) {
 	    killTimer( d->tripleClickTimer );
@@ -1532,18 +1538,17 @@ void QLineEdit::keyPressEvent( QKeyEvent * e )
 {
     d->setCursorVisible( TRUE );
     if ( e->key() == Key_Enter || e->key() == Key_Return ) {
-#ifdef QT_NO_VALIDATOR
-	emit returnPressed();
-#else
 	const QValidator * v = d->validator;
-	if ( !v || v->validate( d->text, d->cursor ) == QValidator::Acceptable ) {
+	if ( hasAcceptableInput() ) {
 	    emit returnPressed();
-	} else {
+	}
+#ifndef QT_NO_VALIDATOR
+	else if ( v && v->validate( d->text, d->cursor ) != QValidator::Acceptable ) {
 	    QString vstr = d->text;
 	    v->fixup( vstr );
 	    if ( vstr != d->text ) {
 		setText( vstr );
-		if ( v->validate( d->text, d->cursor ) == QValidator::Acceptable )
+		if ( hasAcceptableInput() )
 		    emit returnPressed();
 	    }
 	}
@@ -1756,6 +1761,7 @@ void QLineEdit::keyPressEvent( QKeyEvent * e )
     }
     if ( e->key() == Key_Direction_L || e->key() == Key_Direction_R ) {
 	d->direction = (e->key() == Key_Direction_L) ? QChar::DirL : QChar::DirR;
+        d->updateTextLayout();
 	update();
     }
 
@@ -1782,39 +1788,40 @@ void QLineEdit::imComposeEvent( QIMEvent *e )
 {
     if ( d->readOnly ) {
 	e->ignore();
-    } else {
-	d->text.replace( d->imstart, d->imend - d->imstart, e->text() );
-	d->imend = d->imstart + e->text().length();
-	d->imselstart = d->imstart + e->cursorPos();
-	d->imselend = d->imselstart + e->selectionLength();
-	d->cursor = e->selectionLength() ? d->imend : d->imselend;
-	d->updateTextLayout();
-	update();
+	return;
     }
+    d->text.replace( d->imstart, d->imend - d->imstart, e->text() );
+    d->imend = d->imstart + e->text().length();
+    d->imselstart = d->imstart + e->cursorPos();
+    d->imselend = d->imselstart + e->selectionLength();
+    d->cursor = d->imstart + e->cursorPos();
+    d->updateTextLayout();
+    d->updateMicroFocusHint();
+    update();
 }
 
 /*! \reimp
  */
 void QLineEdit::imEndEvent( QIMEvent *e )
 {
-    if ( d->readOnly ) {
+    if ( d->readOnly) {
 	e->ignore();
-    } else {
-	d->text.remove( d->imstart, d->imend - d->imstart );
-	d->cursor = d->imselstart = d->imselend = d->imend = d->imstart;
-	d->textDirty = TRUE;
-	insert( e->text() );
+	return;
     }
+    d->text.remove( d->imstart, d->imend - d->imstart );
+    d->cursor = d->imselstart = d->imselend = d->imend = d->imstart;
+    d->textDirty = TRUE;
+    insert( e->text() );
 }
 
 /*!\reimp
 */
 
-void QLineEdit::focusInEvent( QFocusEvent* e )
+void QLineEdit::focusInEvent( QFocusEvent* )
 {
-    if ( e->reason() == QFocusEvent::Tab ||
-	 e->reason() == QFocusEvent::Backtab  ||
-	 e->reason() == QFocusEvent::Shortcut )
+    if ( QFocusEvent::reason() == QFocusEvent::Tab ||
+	 QFocusEvent::reason() == QFocusEvent::Backtab  ||
+	 QFocusEvent::reason() == QFocusEvent::Shortcut )
 	d->maskData ? d->moveCursor( d->nextMaskBlank( 0 ) ) : selectAll();
     if ( !d->cursorTimer ) {
 	int cft = QApplication::cursorFlashTime();
@@ -1828,16 +1835,17 @@ void QLineEdit::focusInEvent( QFocusEvent* e )
 /*!\reimp
 */
 
-void QLineEdit::focusOutEvent( QFocusEvent* e )
+void QLineEdit::focusOutEvent( QFocusEvent* )
 {
-    if ( e->reason() != QFocusEvent::ActiveWindow &&
-	 e->reason() != QFocusEvent::Popup )
+    if ( QFocusEvent::reason() != QFocusEvent::ActiveWindow &&
+	 QFocusEvent::reason() != QFocusEvent::Popup )
 	deselect();
     d->setCursorVisible( FALSE );
     if ( d->cursorTimer > 0 )
 	killTimer( d->cursorTimer );
     d->cursorTimer = 0;
-    emit lostFocus();
+    if (QFocusEvent::reason() != QFocusEvent::Popup)
+        emit lostFocus();
 }
 
 /*!\reimp
@@ -1854,15 +1862,20 @@ void QLineEdit::drawContents( QPainter *p )
 	bg = QBrush( cg.background(), *paletteBackgroundPixmap() );
     else if ( !isEnabled() )
 	bg = cg.brush( QColorGroup::Background );
+    QPoint brushOrigin = p->brushOrigin();
     p->save();
     p->setClipRegion( QRegion(cr) - lineRect );
+    p->setBrushOrigin(brushOrigin - backgroundOffset());
     p->fillRect( cr, bg );
     p->restore();
     QSharedDoubleBuffer buffer( p, lineRect.x(), lineRect.y(),
  				lineRect.width(), lineRect.height(),
  				hasFocus() ? QSharedDoubleBuffer::Force : 0 );
     p = buffer.painter();
+    brushOrigin = p->brushOrigin();
+    p->setBrushOrigin(brushOrigin - backgroundOffset());
     p->fillRect( lineRect, bg );
+    p->setBrushOrigin(brushOrigin);
 
     // locate cursor position
     int cix = 0;
@@ -1881,7 +1894,7 @@ void QLineEdit::drawContents( QPainter *p )
     if ( (minLB + widthUsed) <=  lineRect.width() ) {
 	switch ( d->visualAlignment() ) {
 	case AlignRight:
-	    d->hscroll = widthUsed - lineRect.width();
+	    d->hscroll = widthUsed - lineRect.width() + 1;
 	    break;
 	case AlignHCenter:
 	    d->hscroll = ( widthUsed - lineRect.width() ) / 2;
@@ -1897,6 +1910,8 @@ void QLineEdit::drawContents( QPainter *p )
 	d->hscroll = cix;
     } else if ( widthUsed - d->hscroll < lineRect.width() ) {
 	d->hscroll = widthUsed - lineRect.width() + 1;
+    } else if (d->hscroll < 0) {
+        d->hscroll = 0;
     }
     // the y offset is there to keep the baseline constant in case we have script changes in the text.
     QPoint topLeft = lineRect.topLeft() - QPoint(d->hscroll, d->ascent-fm.ascent());
@@ -1964,6 +1979,7 @@ void QLineEdit::drawContents( QPainter *p )
 	    p->setPen( paletteBackgroundColor() );
 	    p->drawTextItem( topLeft, ti, textflags );
 	    p->restore();
+            supressCursor = TRUE;
 	}
 
 	// overwrite cursor
@@ -2065,13 +2081,19 @@ void QLineEdit::dropEvent( QDropEvent* e )
     }
 }
 
-void QLineEditPrivate::drag()
+bool QLineEditPrivate::drag()
 {
     q->killTimer( dndTimer );
     dndTimer = 0;
     QTextDrag *tdo = new QTextDrag( q->selectedText(), q );
+
+    QGuardedPtr<QLineEdit> gptr = q;
+    bool r = tdo->drag();
+    if ( !gptr )
+        return FALSE;
+
     // ### fix the check QDragObject::target() != q in Qt4 (should not be needed)
-    if ( tdo->drag() && !readOnly && QDragObject::target() != q ) {
+    if ( r && !readOnly && QDragObject::target() != q ) {
 	int priorState = undoState;
 	removeSelectedText();
 	finishChange( priorState );
@@ -2079,6 +2101,7 @@ void QLineEditPrivate::drag()
 #ifndef QT_NO_CURSOR
     q->setCursor( readOnly ? arrowCursor : ibeamCursor );
 #endif
+    return TRUE;
 }
 
 #endif // QT_NO_DRAGANDDROP
@@ -2091,8 +2114,10 @@ void QLineEdit::contextMenuEvent( QContextMenuEvent * e )
 {
 #ifndef QT_NO_POPUPMENU
     d->separate();
-
-    QGuardedPtr<QPopupMenu> popup = createPopupMenu();
+    QPopupMenu *menu = createPopupMenu();
+    if (!menu)
+        return;
+    QGuardedPtr<QPopupMenu> popup = menu;
     QGuardedPtr<QLineEdit> that = this;
     QPoint pos = e->reason() == QContextMenuEvent::Mouse ? e->globalPos() :
 		 mapToGlobal( QPoint(e->pos().x(), 0) ) + QPoint( width() / 2, height() / 2 );
@@ -2265,13 +2290,13 @@ void QLineEditPrivate::updateTextLayout()
 	    uc[i] = QChar(0x0020);
     }
     textLayout.setText( str, q->font() );
-    // ### want to do textLayout.setRightToLeft( text.isRightToLeft() );
-    textLayout.beginLayout( QTextLayout::SingleLine );
+    textLayout.setDirection((QChar::Direction)direction);
+    textLayout.beginLayout(QTextLayout::SingleLine);
     textLayout.beginLine( INT_MAX );
     while ( !textLayout.atEnd() )
 	textLayout.addCurrentItem();
     ascent = 0;
-    textLayout.endLine(0, 0, Qt::AlignLeft, &ascent);
+    textLayout.endLine(0, 0, Qt::AlignLeft|Qt::SingleLine, &ascent);
 }
 
 int QLineEditPrivate::xToPos( int x, QTextItem::CursorPosition betweenOrOn ) const
@@ -2330,7 +2355,7 @@ void QLineEditPrivate::moveCursor( int pos, bool mark )
 	selstart = QMIN( anchor, pos );
 	selend = QMAX( anchor, pos );
     } else {
-	selstart = selend = 0;
+	deselect();
     }
     if ( fullUpdate ) {
 	cursor = pos;
@@ -2341,9 +2366,10 @@ void QLineEditPrivate::moveCursor( int pos, bool mark )
 	setCursorVisible( TRUE );
     }
     updateMicroFocusHint();
-    if ( mark ) {
-	if( !q->style().styleHint( QStyle::SH_BlinkCursorWhenTextSelected ))
-	    setCursorVisible( FALSE );
+    if ( mark && !q->style().styleHint( QStyle::SH_BlinkCursorWhenTextSelected ) )
+	setCursorVisible( FALSE );
+    if ( mark || selDirty ) {
+	selDirty = FALSE;
 	emit q->selectionChanged();
     }
 }
@@ -2506,10 +2532,12 @@ void QLineEditPrivate::removeSelectedText()
 void QLineEditPrivate::parseInputMask( const QString &maskFields )
 {
     if ( maskFields.isEmpty() || maskFields.section( ';', 0, 0 ).isEmpty() ) {
-	delete [] maskData;
-	maskData = 0;
-	maxLength = 32767;
-	q->setText( QString::null );
+	if ( maskData ) {
+	    delete [] maskData;
+	    maskData = 0;
+	    maxLength = 32767;
+	    q->setText( QString::null );
+	}
 	return;
     }
 
@@ -2702,7 +2730,16 @@ QString QLineEditPrivate::maskString( uint pos, const QString &str, bool clear)
 			n = findInMask( i, TRUE, FALSE, str[(int)strIndex] );
 			if ( n != -1 ) {
 			    s += fill.mid( i, n-i );
-			    s += str[(int)strIndex];
+			    switch ( maskData[ n ].caseMode ) {
+			    case MaskInputData::Upper:
+				s += str[(int)strIndex].upper();
+				break;
+			    case MaskInputData::Lower:
+				s += str[(int)strIndex].lower();
+				break;
+			    default:
+				s += str[(int)strIndex];
+			    }
 			    i = n + 1; // updates i to find + 1
 			}
 		    }
