@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qmenubar.cpp   3.2.3   edited Aug 19 08:33 $
+** $Id: qt/qmenubar.cpp   3.3.6   edited Jan 27 15:36 $
 **
 ** Implementation of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -296,7 +296,7 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 
     setGeometry( 0, 0, width(), h );
 
-    setMouseTracking( style().styleHint(QStyle::SH_GUIStyle) == WindowsStyle );
+    setMouseTracking( style().styleHint(QStyle::SH_MenuBar_MouseTracking) );
 }
 
 
@@ -305,7 +305,7 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 
 void QMenuBar::styleChange( QStyle& old )
 {
-    setMouseTracking( style().styleHint(QStyle::SH_GUIStyle) == WindowsStyle );
+    setMouseTracking( style().styleHint(QStyle::SH_MenuBar_MouseTracking) );
     QFrame::styleChange( old );
 }
 
@@ -360,8 +360,8 @@ void QMenuBar::menuContentsChanged()
     if( pendingDelayedContentsChanges )
         return;
     pendingDelayedContentsChanges = 1;
-    if( !pendingDelayedStateChanges ) // if the timer hasn't been started yet
-	QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
+    if( !pendingDelayedStateChanges )// if the timer hasn't been started yet
+        QTimer::singleShot( 0, this, SLOT(performDelayedChanges()));
 }
 
 void QMenuBar::performDelayedContentsChanged()
@@ -389,24 +389,6 @@ void QMenuBar::performDelayedContentsChanged()
 	    parentWidget()->layout()->activate();
 #endif
     }
-#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
-    if(mac_eaten_menubar) {
-	macDirtyNativeMenubar();
-
-	bool all_hidden = TRUE;
-	if(irects) {
-	    for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
-		all_hidden = irects[i].isEmpty();
-	}
-	if( all_hidden ) {
-	    if( !isHidden())
-		hide();
-	} else {
-	    if( !isShown() && !fromFrameChange )
-		show();
-	}
-    }
-#endif
 }
 
 /*!
@@ -440,10 +422,32 @@ void QMenuBar::performDelayedStateChanged()
 
 void QMenuBar::performDelayedChanges()
 {
+#if defined(Q_WS_MAC) && !defined(QMAC_MENUBAR_NO_NATIVE)
+    // I must do this here as the values change in the function below.
+    bool needMacUpdate = (pendingDelayedContentsChanges || pendingDelayedStateChanges);
+#endif
     if( pendingDelayedContentsChanges )
         performDelayedContentsChanged();
     if( pendingDelayedStateChanges )
         performDelayedStateChanged();
+#if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
+    if(mac_eaten_menubar && needMacUpdate) {
+	macDirtyNativeMenubar();
+
+	bool all_hidden = TRUE;
+	if(irects) {
+	    for(int i = 0; all_hidden && i < (int)mitems->count(); i++)
+		all_hidden = irects[i].isEmpty();
+	}
+	if( all_hidden ) {
+	    if( !isHidden())
+		hide();
+	} else {
+	    if( !isShown() && !fromFrameChange )
+		show();
+	}
+    }
+#endif
 }
 
 
@@ -478,7 +482,7 @@ void QMenuBar::frameChanged()
 
 void QMenuBar::languageChange()
 {
-    menuContentsChanged(); 
+    menuContentsChanged();
 }
 
 /*!
@@ -524,6 +528,12 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	// NOTE: this event came from the real focus widget, so we don't
 	// need to touch the event filters
 	waitforalt = 0;
+	// although the comment above said not to remove the event filter, it is
+	// incorrect. We need to remove our self fom the focused widget as normally
+	// this happens in the key release but it does not happen in this case
+	QWidget * f = ((QWidget *)object)->focusWidget();
+	if (f)
+	    f->removeEventFilter( this );
 	return FALSE;
     } else if ( !( event->type() == QEvent::Accel ||
 		event->type() == QEvent::AccelOverride ||
@@ -899,6 +909,8 @@ int QMenuBar::calculateRects( int max_width )
     if ( gs == MotifStyle ) {
 	x += motifBarHMargin;
 	y += motifBarVMargin;
+    } else if ( style().inherits("QWindowsXPStyle") && style().styleHint(QStyle::SH_TitleBar_NoBorder) ) {
+        ;
     } else if ( gs == WindowsStyle ) {
 	x += 2;
 	y += 2;
@@ -908,6 +920,8 @@ int QMenuBar::calculateRects( int max_width )
 
     int i = 0;
     int separator = -1;
+    const int itemSpacing = style().pixelMetric(QStyle::PM_MenuBarItemSpacing);
+    const int lastItem = reverse ? 0 : mitems->count() - 1;
 
     while ( i < (int)mitems->count() ) {	// for each menu item...
 	QMenuItem *mi = mitems->at(i);
@@ -946,7 +960,7 @@ int QMenuBar::calculateRects( int max_width )
 #if defined(Q_WS_MAC) && !defined(QMAC_QMENUBAR_NO_NATIVE)
 	    if ( !mac_eaten_menubar ) {
 #endif
-		if ( gs == MotifStyle ) {
+		if ( gs == MotifStyle && mi->isVisible() ) {
 		    w += 2*motifItemFrame;
 		    h += 2*motifItemFrame;
 		}
@@ -955,7 +969,7 @@ int QMenuBar::calculateRects( int max_width )
 #endif
 
 	    if ( ( ( !reverse && x + w + frameWidth() - max_width > 0 ) ||
-		 ( reverse && x - w -frameWidth() < 0 ) )
+		 ( reverse && x - w - itemSpacing - frameWidth() < 0 ) )
 		 && nlitems > 0 ) {
 		nlitems = 0;
 		x = frameWidth();
@@ -965,7 +979,7 @@ int QMenuBar::calculateRects( int max_width )
 		    y += motifBarVMargin;
 		}
 		if ( reverse )
-		    x = max_width - x;
+		    x = max_width - x + itemSpacing;
 		if ( style().styleHint(QStyle::SH_GUIStyle) == MotifStyle )
 		    separator = -1;
 	    }
@@ -974,13 +988,22 @@ int QMenuBar::calculateRects( int max_width )
 	    if ( h > max_item_height )
 		max_item_height = h;
 	}
-	if( reverse )
+
+	const bool isLast = (i == lastItem);
+
+	if( reverse ) {
 	    x -= w;
+	    if (!isLast && !mi->isSeparator())
+		x -= itemSpacing;
+	}
 	if ( update ) {
 	    irects[i].setRect( x, y, w, h );
 	}
-	if ( !reverse )
+	if ( !reverse ) {
 	    x += w;
+	    if (!isLast && !mi->isSeparator())
+		x += itemSpacing;
+	}
 	nlitems++;
 	i++;
     }
@@ -1049,12 +1072,18 @@ QRect QMenuBar::itemRect( int index )
   it is a separator item.
 */
 
-int QMenuBar::itemAtPos( const QPoint &pos )
+int QMenuBar::itemAtPos( const QPoint &pos_ )
 {
     calculateRects();
     if ( !irects )
 	return -1;
     int i = 0;
+    QPoint pos = pos_;
+    // Fitts' Law for edges - compensate for the extra margin
+    // added in calculateRects()
+    const int margin = 2;
+    pos.setX( QMAX( margin, QMIN( width() - margin, pos.x())));
+    pos.setY( QMAX( margin, QMIN( height() - margin, pos.y())));
     while ( i < (int)mitems->count() ) {
 	if ( !irects[i].isEmpty() && irects[i].contains( pos ) ) {
 	    QMenuItem *mi = mitems->at(i);
@@ -1200,8 +1229,10 @@ void QMenuBar::mouseReleaseEvent( QMouseEvent *e )
     if ( toggleclose &&
 	 // pressing an item twice closes in windows, but not in motif :/
 	 style().styleHint(QStyle::SH_GUIStyle) == WindowsStyle &&
-	 actItem == item )
+	 actItem == item ) {
 	showMenu = FALSE;
+	setAltMode( FALSE );
+    }
     setActiveItem( item, showMenu, !hasMouseTracking() );
     toggleclose = 0;
 }
@@ -1302,7 +1333,7 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 		break;
 	}
 	setActiveItem( i, popupvisible	 );
-    } else if ( ( !e->state() || ( e->state() & (MetaButton|AltButton) ) ) && e->text().length()==1 ) {
+    } else if ( ( !e->state() || (e->state()&(MetaButton|AltButton)) ) && e->text().length()==1 && !popupvisible ) {
 	QChar c = e->text()[0].upper();
 
 	QMenuItemListIt it(*mitems);
