@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qscrollbar.cpp   3.0.5   edited Jun 3 11:47 $
+** $Id: qt/qscrollbar.cpp   3.3.6   edited Aug 31 2005 $
 **
 ** Implementation of QScrollBar class
 **
 ** Created : 940427
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -51,130 +51,156 @@
 #include <limits.h>
 
 /*!
-  \class QScrollBar qscrollbar.h
+    \class QScrollBar
+    \brief The QScrollBar widget provides a vertical or horizontal scroll bar.
 
-  \brief The QScrollBar widget provides a vertical or horizontal scroll bar.
+    \ingroup basic
 
-  \ingroup basic
+    A scroll bar allows the user to control a value within a
+    program-definable range and gives users a visible indication of
+    the current value of a \link QRangeControl range control \endlink.
 
-  A scroll bar allows the user to control a value within a
-  program-definable range and gives users a visible indication of
-  the current value of a \link QRangeControl range control \endlink.
+    Scroll bars include four separate controls:
 
-  Scroll bars include four separate controls:
-  \list
-   \i The \e line-up and \e line-down controls are little buttons
-    which the user can use to move one line up or down.  The meaning of "line"
-    is configurable.  In editors and list boxes it means one line of text;
-    in an image viewer it might mean 20 pixels.
-   \i The \e slider is the handle that indicates the current value
-    of the scroll bar, which the user can drag to change the value.
-    This part of the scroll bar is sometimes called the "thumb".
-   \i The \e page-up/page-down control is the area on which the slider
-    slides (the scroll bar's background).  Clicking here moves the scroll
-    bar towards the click.  The meaning of "page" is also
+    \list
+
+    \i The \e line-up and \e line-down controls are little buttons
+    which the user can use to move one "line" up or down. The meaning
+    of line is configurable. In editors and list boxes it means one
+    line of text; in an image viewer it might mean 20 pixels.
+
+    \i The \e slider is the handle that indicates the current value of
+    the scroll bar, which the user can drag to change the value. This
+    part of the scroll bar is sometimes called the "thumb".
+
+    \i The \e page-up/page-down control is the area on which the
+    slider slides (the scroll bar's background). Clicking here moves
+    the scroll bar towards the click. The meaning of "page" is also
     configurable: in editors and list boxes it means as many lines as
     there is space for in the widget.
-  \endlist
 
-  QScrollBar has very few of its own functions; it mostly relies on
-  QRangeControl.  The most useful functions are setValue() to set the
-  scroll bar directly to some value; addPage(), addLine(), subtractPage(),
-  and subtractLine() to simulate the effects of clicking (useful for
-  accelerator keys); setSteps() to define the values of pageStep() and
-  lineStep(); and setRange() to set the minValue()
-  and maxValue() of the scroll bar.  QScrollBar has a convenience
-  constructor with which you can set most of these properties.
+    \endlist
 
-  Some GUI styles (for example, the Windows and Motif styles provided),
-  also use the pageStep() value to calculate the size of the slider.
+    QScrollBar has very few of its own functions; it mostly relies on
+    QRangeControl. The most useful functions are setValue() to set the
+    scroll bar directly to some value; addPage(), addLine(),
+    subtractPage(), and subtractLine() to simulate the effects of
+    clicking (useful for accelerator keys); setSteps() to define the
+    values of pageStep() and lineStep(); and setRange() to set the
+    minValue() and maxValue() of the scroll bar. QScrollBar has a
+    convenience constructor with which you can set most of these
+    properties.
 
-  In addition to the access functions from QRangeControl, QScrollBar
-  has a comprehensive set of signals:
-  \list
-  \i valueChanged() - emitted when the scroll bar's value has changed.
-	  The tracking() determines whether this signal is emitted
-	  during user interaction.
-  \i sliderPressed() - emitted when the user starts to drag the
-	  slider.
-  \i sliderMoved() - emitted when the user drags the slider.
-  \i sliderReleased() - emitted when the user releases the slider.
-  \i nextLine() - emitted when the scroll bar has moved one line
-	  down or right.  Line is defined in QRangeControl.
-  \i prevLine() - emitted when the scroll bar has moved one line
-	  up or left.
-  \i nextPage() - emitted when the scroll bar has moved one page
-	  down or right.
-  \i prevPage() - emitted when the scroll bar has moved one page
-      up or left.
-  \endlist
+    Some GUI styles (for example, the Windows and Motif styles
+    provided with Qt), also use the pageStep() value to calculate the
+    size of the slider.
 
-  QScrollBar only provides integer ranges.  Note that although QScrollBar
-  handles very large numbers, scroll bars on current screens cannot
-  usefully control ranges above about 100,000 pixels.  Beyond that,
-  it becomes difficult for the user to control the
-  scroll bar using either the keyboard or the mouse.
+    In addition to the access functions from QRangeControl, QScrollBar
+    provides a comprehensive set of signals:
+    \table
+    \header \i Signal \i Emitted when
+    \row \i \l valueChanged()
+	 \i the scroll bar's value has changed. The tracking()
+	    determines whether this signal is emitted during user
+	    interaction.
+    \row \i \l sliderPressed()
+	 \i the user starts to drag the slider.
+    \row \i \l sliderMoved()
+	 \i the user drags the slider.
+    \row \i \l sliderReleased()
+	 \i the user releases the slider.
+    \row \i \l nextLine()
+	 \i the scroll bar has moved one line down or right. Line is
+	    defined in QRangeControl.
+    \row \i \l prevLine()
+	 \i the scroll bar has moved one line up or left.
+    \row \i \l nextPage()
+	 \i the scroll bar has moved one page down or right.
+    \row \i \l prevPage()
+	 \i the scroll bar has moved one page up or left.
+    \endtable
 
-  A scroll bar can be controlled by the keyboard, but it has a
-  default focusPolicy() of \c NoFocus. Use setFocusPolicy() to
-  enable keyboard focus. See keyPressEvent() for a list of
-  key bindings.
+    QScrollBar only provides integer ranges. Note that although
+    QScrollBar handles very large numbers, scroll bars on current
+    screens cannot usefully control ranges above about 100,000 pixels.
+    Beyond that, it becomes difficult for the user to control the
+    scroll bar using either the keyboard or the mouse.
 
-  If you need to add scroll bars to an interface, consider using the
-  QScrollView class, which encapsulates the common uses for scroll bars.
+    A scroll bar can be controlled by the keyboard, but it has a
+    default focusPolicy() of \c NoFocus. Use setFocusPolicy() to
+    enable keyboard focus. See keyPressEvent() for a list of key
+    bindings.
 
-  <img src=qscrbar-m.png> <img src=qscrbar-w.png>
+    If you need to add scroll bars to an interface, consider using the
+    QScrollView class, which encapsulates the common uses for scroll
+    bars.
 
-  \sa QSlider QSpinBox QScrollView
-  \link guibooks.html#fowler GUI Design Handbook: Scroll Bar\endlink
+    <img src=qscrbar-m.png> <img src=qscrbar-w.png>
+
+    \sa QSlider QSpinBox QScrollView
+    \link guibooks.html#fowler GUI Design Handbook: Scroll Bar\endlink
 */
 
 
 /*!
-  \fn void QScrollBar::valueChanged( int value )
-  This signal is emitted when the scroll bar value has changed, with the
-  new scroll bar \a value as an argument.
+    \fn void QScrollBar::valueChanged( int value )
+
+    This signal is emitted when the scroll bar value has changed, with
+    the new scroll bar \a value as an argument.
 */
 
 /*!
-  \fn void QScrollBar::sliderPressed()
-  This signal is emitted when the user presses the slider with the mouse.
+    \fn void QScrollBar::sliderPressed()
+
+    This signal is emitted when the user presses the slider with the
+    mouse.
 */
 
 /*!
-  \fn void QScrollBar::sliderMoved( int value )
+    \fn void QScrollBar::sliderMoved( int value )
 
-  This signal is emitted when the slider is moved by the user, with
-  the new scroll bar \a value as an argument.
+    This signal is emitted when the slider is dragged by the user, with
+    the new scroll bar \a value as an argument.
 
-  This signal is emitted even when tracking is turned off.
+    This signal is emitted even when tracking is turned off.
 
-  \sa tracking() valueChanged() nextLine() prevLine() nextPage() prevPage()
+    \sa tracking() valueChanged() nextLine() prevLine() nextPage()
+    prevPage()
 */
 
 /*!
-  \fn void QScrollBar::sliderReleased()
-  This signal is emitted when the user releases the slider with the mouse.
+    \fn void QScrollBar::sliderReleased()
+
+    This signal is emitted when the user releases the slider with the
+    mouse.
 */
 
 /*!
-  \fn void QScrollBar::nextLine()
-  This signal is emitted when the scroll bar scrolls one line down or right.
+    \fn void QScrollBar::nextLine()
+
+    This signal is emitted when the scroll bar scrolls one line down
+    or right.
 */
 
 /*!
-  \fn void QScrollBar::prevLine()
-  This signal is emitted when the scroll bar scrolls one line up or left.
+    \fn void QScrollBar::prevLine()
+
+    This signal is emitted when the scroll bar scrolls one line up or
+    left.
 */
 
 /*!
-  \fn void QScrollBar::nextPage()
-  This signal is emitted when the scroll bar scrolls one page down or right.
+    \fn void QScrollBar::nextPage()
+
+    This signal is emitted when the scroll bar scrolls one page down
+    or right.
 */
 
 /*!
-  \fn void QScrollBar::prevPage()
-  This signal is emitted when the scroll bar scrolls one page up or left.
+    \fn void QScrollBar::prevPage()
+
+    This signal is emitted when the scroll bar scrolls one page up or
+    left.
 */
 
 
@@ -189,56 +215,73 @@ static const int repeatTime	= 50;
 
 
 /*!
-  Constructs a vertical scroll bar.
+    Constructs a vertical scroll bar.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
+
+    The \c minValue defaults to 0, the \c maxValue to 99, with a \c
+    lineStep size of 1 and a \c pageStep size of 10, and an initial
+    \c value of 0.
 */
 
 QScrollBar::QScrollBar( QWidget *parent, const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name ), orient( Vertical )
 {
-    orient = Vertical;
     init();
 }
 
 /*!
-  Constructs a scroll bar.
+    Constructs a scroll bar.
 
-  The \a orientation must be Qt::Vertical or Qt::Horizontal.
+    The \a orientation must be \c Qt::Vertical or \c Qt::Horizontal.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
+
+    The \c minValue defaults to 0, the \c maxValue to 99, with a \c
+    lineStep size of 1 and a \c pageStep size of 10, and an initial
+    \c value of 0.
 */
 
 QScrollBar::QScrollBar( Orientation orientation, QWidget *parent,
 			const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name ), orient( orientation )
 {
-    orient = orientation;
     init();
 }
 
 /*!
-  Constructs a scroll bar whose value can never be smaller than \a
-  minValue or greater than \a maxValue, whose line step size is \a
-  lineStep and page step size is \a pageStep and whose value is
-  initially \a value (which is guaranteed to be in range using bound()).
+    Constructs a scroll bar whose value can never be smaller than \a
+    minValue or greater than \a maxValue, whose line step size is \a
+    lineStep and page step size is \a pageStep and whose value is
+    initially \a value (which is guaranteed to be in range using
+    bound()).
 
-  If \a orientation is \c Vertical the scroll bar is vertical and if it
-  is \c Horizontal the scroll bar is horizontal.
+    If \a orientation is \c Vertical the scroll bar is vertical and if
+    it is \c Horizontal the scroll bar is horizontal.
 
-  The \a parent and \a name arguments are sent to the QWidget constructor.
+    The \a parent and \a name arguments are sent on to the QWidget
+    constructor.
 */
 
 QScrollBar::QScrollBar( int minValue, int maxValue, int lineStep, int pageStep,
 			int value,  Orientation orientation,
 			QWidget *parent, const char *name )
     : QWidget( parent, name ),
-      QRangeControl( minValue, maxValue, lineStep, pageStep, value )
+      QRangeControl( minValue, maxValue, lineStep, pageStep, value ),
+      orient( orientation )
 {
-    orient = orientation;
     init();
 }
 
+/*!
+    Destructor.
+*/
+QScrollBar::~QScrollBar()
+{
+}
+
 void QScrollBar::init()
 {
     track = TRUE;
@@ -248,55 +291,59 @@ void QScrollBar::init()
     setFocusPolicy( NoFocus );
 
     repeater = 0;
-    d = 0;
 
     setBackgroundMode((Qt::BackgroundMode)
 		      style().styleHint(QStyle::SH_ScrollBar_BackgroundMode));
 
-    if ( orient == Horizontal )
-	setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
-    else
-	setSizePolicy( QSizePolicy(  QSizePolicy::Fixed, QSizePolicy::Minimum ) );
+    QSizePolicy sp( QSizePolicy::Minimum, QSizePolicy::Fixed );
+    if ( orient == Vertical )
+	sp.transpose();
+    setSizePolicy( sp );
+    clearWState( WState_OwnSizePolicy );
 }
 
 
 /*!
-  \property QScrollBar::orientation
-  \brief the orientation of the scroll bar
+    \property QScrollBar::orientation
+    \brief the orientation of the scroll bar
 
-  The orientation must be \l Qt::Vertical (the default) or \l Qt::Horizontal.
+    The orientation must be \l Qt::Vertical (the default) or \l
+    Qt::Horizontal.
 */
 
 void QScrollBar::setOrientation( Orientation orientation )
 {
     if ( orientation == orient )
 	return;
-
-    QSizePolicy pol = sizePolicy();
-    setSizePolicy( QSizePolicy( pol.verData(), pol.horData() ) );
+    if ( !testWState( WState_OwnSizePolicy ) ) {
+	QSizePolicy sp = sizePolicy();
+	sp.transpose();
+	setSizePolicy( sp );
+	clearWState( WState_OwnSizePolicy );
+    }
 
     orient = orientation;
 
     positionSliderFromValue();
     update();
+    updateGeometry();
 }
 
 /*!
-  \property QScrollBar::tracking
-  \brief whether scroll bar tracking is enabled
+    \property QScrollBar::tracking
+    \brief whether scroll bar tracking is enabled
 
-  If tracking is enabled (the default), the scroll bar emits the
-  valueChanged() signal while the slider is being dragged.  If
-  tracking is disabled, the scroll bar emits the valueChanged() signal
-  only when the user releases the mouse button after moving the
-  slider.
+    If tracking is enabled (the default), the scroll bar emits the
+    valueChanged() signal while the slider is being dragged. If
+    tracking is disabled, the scroll bar emits the valueChanged()
+    signal only when the user releases the mouse button after moving
+    the slider.
 */
 
 
 /*!
-  \property QScrollBar::draggingSlider
-  \brief whether the user has clicked the mouse on the slider and
-  is currently dragging it
+    \property QScrollBar::draggingSlider
+    \brief whether the user has clicked the mouse on the slider and is currently dragging it
 */
 
 bool QScrollBar::draggingSlider() const
@@ -306,9 +353,10 @@ bool QScrollBar::draggingSlider() const
 
 
 /*!
-  Reimplements the virtual function QWidget::setPalette().
+    Reimplements the virtual function QWidget::setPalette().
 
-  Sets the background color to the mid color for Motif style scroll bars using palette \a p.
+    Sets the background color to the mid color for Motif style scroll
+    bars using palette \a p.
 */
 
 void QScrollBar::setPalette( const QPalette &p )
@@ -319,8 +367,7 @@ void QScrollBar::setPalette( const QPalette &p )
 }
 
 
-/*!\reimp
-*/
+/*! \reimp */
 QSize QScrollBar::sizeHint() const
 {
     constPolish();
@@ -333,6 +380,16 @@ QSize QScrollBar::sizeHint() const
     }
 }
 
+/*! \fn void QScrollBar::setSizePolicy( QSizePolicy::SizeType, QSizePolicy::SizeType, bool )
+    \reimp
+*/
+
+/*! \reimp */
+void QScrollBar::setSizePolicy( QSizePolicy sp )
+{
+    //## remove 4.0
+    QWidget::setSizePolicy( sp );
+}
 
 /*!
   \internal
@@ -343,7 +400,7 @@ void QScrollBar::valueChange()
 {
     int tmp = sliderPos;
     positionSliderFromValue();
-    if ( tmp != sliderPos && isVisibleTo( 0 ) )
+    if ( tmp != sliderPos && isVisible() )
 	drawControls(QStyle::SC_ScrollBarAddPage |
 		     QStyle::SC_ScrollBarSubPage |
 		     QStyle::SC_ScrollBarSlider,
@@ -373,7 +430,7 @@ void QScrollBar::rangeChange()
 {
     positionSliderFromValue();
 
-    if ( isVisibleTo( 0 ) )
+    if ( isVisible() )
 	drawControls(QStyle::SC_ScrollBarAddLine |
 		     QStyle::SC_ScrollBarSubLine |
 		     QStyle::SC_ScrollBarAddPage |
@@ -386,7 +443,7 @@ void QScrollBar::rangeChange()
 
 
 /*!
-  Handles timer events for the scroll bar.
+    Handles timer events for the scroll bar.
 */
 
 void QScrollBar::doAutoRepeat()
@@ -411,9 +468,10 @@ void QScrollBar::doAutoRepeat()
 }
 
 
-/*! Starts the auto-repeat logic.  Some time after this function is
-called, the auto-repeat starts taking effect and from then on repeats
-until stopAutoRepeat() is called.
+/*!
+    Starts the auto-repeat logic. Some time after this function is
+    called, the auto-repeat starts taking effect and from then on
+    repeats until stopAutoRepeat() is called.
 */
 
 void QScrollBar::startAutoRepeat()
@@ -427,7 +485,9 @@ void QScrollBar::startAutoRepeat()
 }
 
 
-/*! Stops the auto-repeat logic. */
+/*!
+    Stops the auto-repeat logic.
+*/
 
 void QScrollBar::stopAutoRepeat()
 {
@@ -436,7 +496,8 @@ void QScrollBar::stopAutoRepeat()
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 #ifndef QT_NO_WHEELEVENT
 void QScrollBar::wheelEvent( QWheelEvent *e )
@@ -447,10 +508,12 @@ void QScrollBar::wheelEvent( QWheelEvent *e )
 	offset_owner = this;
 	offset = 0;
     }
+    if ( e->orientation() != orient && !rect().contains(e->pos()) )
+	return;
     e->accept();
     int step = QMIN( QApplication::wheelScrollLines()*lineStep(),
 		     pageStep() );
-    if ( e->state() & ControlButton )
+    if ( ( e->state() & ControlButton ) || ( e->state() & ShiftButton ) )
 	step = pageStep();
     offset += -e->delta()*step/120;
     if (QABS(offset)<1)
@@ -460,7 +523,8 @@ void QScrollBar::wheelEvent( QWheelEvent *e )
 }
 #endif
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::keyPressEvent( QKeyEvent *e )
 {
@@ -512,7 +576,8 @@ void QScrollBar::keyPressEvent( QKeyEvent *e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::resizeEvent( QResizeEvent * )
 {
@@ -520,7 +585,8 @@ void QScrollBar::resizeEvent( QResizeEvent * )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::paintEvent( QPaintEvent * )
 {
@@ -537,15 +603,19 @@ void QScrollBar::paintEvent( QPaintEvent * )
 
 static QCOORD sliderStartPos = 0;
 
-/*!\reimp
+/*!
+    \reimp
  */
 void QScrollBar::contextMenuEvent( QContextMenuEvent *e )
 {
     if(clickedAt)
 	e->consume();
+    else
+	e->ignore();
 }
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::mousePressEvent( QMouseEvent *e )
 {
@@ -569,7 +639,8 @@ void QScrollBar::mousePressEvent( QMouseEvent *e )
     if ( (pressedControl == QStyle::SC_ScrollBarAddPage ||
 	  pressedControl == QStyle::SC_ScrollBarSubPage ||
 	  pressedControl == QStyle::SC_ScrollBarSlider ) &&
-	 midButtonAbsPos && e->button() == MidButton ) {
+	 ((midButtonAbsPos && e->button() == MidButton) ||
+	  style().styleHint(QStyle::SH_ScrollBar_LeftClickAbsolutePosition) && e->button() == LeftButton)) {
 
 	QRect sr = style().querySubControlMetrics(QStyle::CC_ScrollBar, this,
 						  QStyle::SC_ScrollBarSlider ),
@@ -614,12 +685,12 @@ void QScrollBar::mousePressEvent( QMouseEvent *e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::mouseReleaseEvent( QMouseEvent *e )
 {
-    if ( !clickedAt || !( e->button() == LeftButton ||
-			  e->button() == MidButton ) )
+    if ( !clickedAt )
 	return;
 
     if ( e->stateAfter() & MouseButtonMask ) // some other button is still pressed
@@ -650,7 +721,8 @@ void QScrollBar::mouseReleaseEvent( QMouseEvent *e )
 }
 
 
-/*!\reimp
+/*!
+    \reimp
 */
 void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 {
@@ -703,8 +775,6 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 	    newSliderPos = sliderMin;
 	else if ( newSliderPos > sliderMax )
 	    newSliderPos = sliderMax;
-	if ( newSliderPos == sliderPos )
-	    return;
 	int newVal = sliderPosToRangeValue(newSliderPos);
 	if ( newVal != slidePrevVal )
 	    emit sliderMoved( newVal );
@@ -737,16 +807,18 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 
 
 /*!
-  \fn int QScrollBar::sliderStart() const
-  Returns the pixel position where the scroll bar slider starts.
+    \fn int QScrollBar::sliderStart() const
 
-  This is equivalent to sliderRect().y() for vertical
-  scroll bars or sliderRect().x() for horizontal scroll bars.
+    Returns the pixel position where the scroll bar slider starts.
+
+    This is equivalent to sliderRect().y() for vertical scroll bars or
+    sliderRect().x() for horizontal scroll bars.
 */
 
 /*!
-  Returns the scroll bar slider rectangle.
-  \sa sliderStart()
+    Returns the scroll bar slider rectangle.
+
+    \sa sliderStart()
 */
 
 QRect QScrollBar::sliderRect() const
@@ -812,34 +884,34 @@ void QScrollBar::action( int control )
 {
     switch( control ) {
     case QStyle::SC_ScrollBarAddLine:
-	emit nextLine();
 	addLine();
+	emit nextLine();
 	break;
     case QStyle::SC_ScrollBarSubLine:
-	emit prevLine();
 	subtractLine();
+	emit prevLine();
 	break;
     case QStyle::SC_ScrollBarAddPage:
-	emit nextPage();
 	addPage();
+	emit nextPage();
 	break;
     case QStyle::SC_ScrollBarSubPage:
-	emit prevPage();
 	subtractPage();
+	emit prevPage();
 	break;
     case QStyle::SC_ScrollBarFirst:
-	emit valueChanged( minValue() );
+	setValue( minValue() );
 #if defined(QT_ACCESSIBILITY_SUPPORT)
 	QAccessible::updateAccessibility( this, 0, QAccessible::ValueChanged );
 #endif
-	setValue( minValue() );
+	emit valueChanged( minValue() );
 	break;
     case QStyle::SC_ScrollBarLast:
-	emit valueChanged( maxValue() );
+	setValue( maxValue() );
 #if defined(QT_ACCESSIBILITY_SUPPORT)
 	QAccessible::updateAccessibility( this, 0, QAccessible::ValueChanged );
 #endif
-	setValue( maxValue() );
+	emit valueChanged( maxValue() );
 	break;
     default:
 	break;
@@ -873,8 +945,9 @@ void QScrollBar::drawControls( uint controls, uint activeControl,
 			       (QStyle::SubControl) activeControl );
 }
 
-/*!\reimp
- */
+/*!
+    \reimp
+*/
 void QScrollBar::styleChange( QStyle& old )
 {
     positionSliderFromValue();
@@ -884,13 +957,13 @@ void QScrollBar::styleChange( QStyle& old )
 }
 
 /*!
-  \property QScrollBar::minValue
-  \brief the current minimum value of the scroll bar
+    \property QScrollBar::minValue
+    \brief the scroll bar's minimum value
 
-  When setting this property, the \l QScrollBar::maxValue is adjusted
-  so that the range remains valid if necessary.
+    When setting this property, the \l QScrollBar::maxValue is
+    adjusted if necessary to ensure that the range remains valid.
 
-  \sa setRange()
+    \sa setRange()
 */
 int QScrollBar::minValue() const
 {
@@ -903,13 +976,13 @@ void QScrollBar::setMinValue( int minVal )
 }
 
 /*!
-  \property QScrollBar::maxValue
-  \brief the current maximum value of the scroll bar
+    \property QScrollBar::maxValue
+    \brief the scroll bar's maximum value
 
-  When setting this property, the \l QScrollBar::minValue is adjusted
-  so that the range remains valid if necessary.
+    When setting this property, the \l QScrollBar::minValue is
+    adjusted if necessary to ensure that the range remains valid.
 
-  \sa setRange()
+    \sa setRange()
 */
 int QScrollBar::maxValue() const
 {
@@ -922,13 +995,14 @@ void QScrollBar::setMaxValue( int maxVal )
 }
 
 /*!
-  \property QScrollBar::lineStep
-  \brief the current line step
+    \property QScrollBar::lineStep
+    \brief the line step
 
-  When setting lineStep, the virtual stepChange() function will be called
-  if the new line step is different from the previous setting.
+    When setting lineStep, the virtual stepChange() function will be
+    called if the new line step is different from the previous
+    setting.
 
-  \sa setSteps() QRangeControl::pageStep() setRange()
+    \sa setSteps() QRangeControl::pageStep() setRange()
 */
 
 int QScrollBar::lineStep() const
@@ -937,13 +1011,14 @@ int QScrollBar::lineStep() const
 }
 
 /*!
-  \property QScrollBar::pageStep
-  \brief the current page step
+    \property QScrollBar::pageStep
+    \brief the page step
 
-  When setting pageStep, the virtual stepChange() function will be called
-  if the new page step is different from the previous setting.
+    When setting pageStep, the virtual stepChange() function will be
+    called if the new page step is different from the previous
+    setting.
 
-  \sa QRangeControl::setSteps() setLineStep() setRange()
+    \sa QRangeControl::setSteps() setLineStep() setRange()
 */
 
 int QScrollBar::pageStep() const
@@ -962,10 +1037,10 @@ void QScrollBar::setPageStep( int i )
 }
 
 /*!
-  \property QScrollBar::value
-  \brief the current scroll bar value
+    \property QScrollBar::value
+    \brief the scroll bar's value
 
-  \sa QRangeControl::value() prevValue()
+    \sa QRangeControl::value() prevValue()
 */
 
 int QScrollBar::value() const
