@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qscrollview.cpp   3.3.5   edited Aug 31 12:13 $
+** $Id: qt/qscrollview.cpp   3.3.6   edited Feb 15 11:49 $
 **
 ** Implementation of QScrollView class
 **
@@ -48,6 +48,7 @@
 #include "qapplication.h"
 #include "qtimer.h"
 #include "qstyle.h"
+#include "qlistview.h"
 #ifdef Q_WS_MAC
 # include "qt_mac.h"
 #endif
@@ -2376,6 +2377,11 @@ void QScrollView::drawContents(QPainter*, int, int, int, int)
 */
 void QScrollView::frameChanged()
 {
+    // slight ugle-hack - the listview header needs readjusting when
+    // changing the frame
+    if (QListView *lv = ::qt_cast<QListView *>(this))
+        lv->triggerUpdate();
+    QFrame::frameChanged();
     updateScrollBars();
 }
 
