@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qspinbox.cpp   3.2.3   edited Jun 12 15:57 $
+** $Id: qt/qspinbox.cpp   3.3.6   edited Jan 10 22:19 $
 **
 ** Implementation of QSpinBox widget class
 **
 ** Created : 970101
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -51,6 +51,17 @@
 #include "qaccessible.h"
 #endif
 
+static bool sumOutOfRange(int current, int add)
+{
+    if (add > 0 && INT_MAX - add < current) {
+        return true;
+    }
+    if (add < 0 && INT_MIN - add > current) {
+        return true;
+    }
+    return false;
+}
+
 class QSpinBoxPrivate
 {
 public:
@@ -561,10 +572,11 @@ void QSpinBox::stepUp()
 {
     if ( edited )
 	interpretText();
-    if ( wrapping() && ( value()+lineStep() > maxValue() ) )
+    if ( wrapping() && ( value()+lineStep() > maxValue() || sumOutOfRange(value(), lineStep() ) ) ) {
 	setValue( minValue() );
-    else
+    } else {
 	addLine();
+    }
 }
 
 
@@ -581,10 +593,11 @@ void QSpinBox::stepDown()
 {
     if ( edited )
 	interpretText();
-    if ( wrapping() && ( value()-lineStep() < minValue() ) )
+    if ( wrapping() && ( value()-lineStep() < minValue() || sumOutOfRange(value(), -lineStep() ) ) ) {
 	setValue( maxValue() );
-    else
+    } else {
 	subtractLine();
+    }
 }
 
 
