@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qtoolbutton.cpp   3.2.3   edited Jun 26 16:21 $
+** $Id: qt/qtoolbutton.cpp   3.3.6   edited Aug 31 2005 $
 **
 ** Implementation of QToolButton class
 **
 ** Created : 980320
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -248,12 +248,12 @@ QToolButton::QToolButton( const QIconSet& iconSet, const QString &textLabel,
     }
 #ifndef QT_NO_TOOLTIP
     if ( !textLabel.isEmpty() ) {
-	if ( !grouptext.isEmpty() )
+	if ( !grouptext.isEmpty() && parent->mainWindow() )
 	    QToolTip::add( this, textLabel,
 			   parent->mainWindow()->toolTipGroup(), grouptext );
 	else
 	    QToolTip::add( this, textLabel );
-    } else if ( !grouptext.isEmpty() )
+    } else if ( !grouptext.isEmpty() && parent->mainWindow() )
 	QToolTip::add( this, QString::null,
 		parent->mainWindow()->toolTipGroup(), grouptext );
 #endif
@@ -277,6 +277,15 @@ QToolButton::~QToolButton()
 }
 
 
+/*!
+    \property QToolButton::backgroundMode
+    \brief the toolbutton's background mode
+
+    Get this property with backgroundMode().
+
+    \sa QWidget::setBackgroundMode()
+*/
+
 /*!
     \property QToolButton::toggleButton
     \brief whether this tool button is a toggle button.
@@ -622,11 +631,12 @@ bool QToolButton::eventFilter( QObject *o, QEvent *e )
 
 bool QToolButton::uses3D() const
 {
-    return !autoRaise() || ( hasMouse() && isEnabled() )
+    return style().styleHint(QStyle::SH_ToolButton_Uses3D)
+	&& (!autoRaise() || ( hasMouse() && isEnabled() )
 #ifndef QT_NO_POPUPMENU
-	|| ( d->popup && d->popup->isVisible() && d->delay <= 0 ) || d->instantPopup
+	    || ( d->popup && d->popup->isVisible() && d->delay <= 0 ) || d->instantPopup
 #endif
-	;
+	    );
 }
 
 
