@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the Qt Assistant.
 **
@@ -210,6 +210,14 @@ void MainWindow::setupGoActions()
 
 }
 
+void MainWindow::browserTabChanged()
+{
+    if (tabs->currentBrowser()) {
+        actionGoPrevious->setEnabled(tabs->currentBrowser()->isBackwardAvailable());
+        actionGoNext->setEnabled(tabs->currentBrowser()->isForwardAvailable());
+    }
+}
+
 bool MainWindow::insertActionSeparator()
 {
     goMenu->insertSeparator();
@@ -239,7 +247,7 @@ void MainWindow::about()
     QMessageBox box( this );
     box.setText( "<center><img src=\"splash.png\">"
 		 "<p>Version " + QString(QT_VERSION_STR) + "</p>"
-		 "<p>Copyright (C) 2000-2003 Trolltech AS. All rights reserved."
+		 "<p>Copyright (C) 2000-2005 Trolltech AS. All rights reserved."
 		 "</p></center><p></p>"
 		 "<p>Qt Commercial Edition license holders: This program is"
 		 " licensed to you under the terms of the Qt Commercial License"
@@ -339,7 +347,7 @@ void MainWindow::print()
 	do {
 	    qApp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
 
-	    richText.draw( &p, margin, margin, view, colorGroup() );
+	    richText.draw( &p, margin, margin, view, palette().active() );
 	    view.moveBy( 0, view.height() );
 	    p.translate( 0 , -view.height() );
 	    p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
