@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -372,7 +372,7 @@ void CustomWidgetEditor::chooseHeader()
     if ( !i || !w )
 	return;
 
-    QString h = QFileDialog::getOpenFileName( QString::null, tr( "Header Files (*.h *.h++ *.hxx)" ), this );
+    QString h = QFileDialog::getOpenFileName( QString::null, tr( "Header Files (*.h *.h++ *.hxx *.hh)" ), this );
     if ( h.isEmpty() )
 	return;
     editHeader->setText( h );
@@ -493,6 +493,7 @@ void CustomWidgetEditor::slotNameChanged( const QString &s )
     MetaDataBase::Function slot;
     slot.function = listSlots->currentItem()->text( 0 );
     slot.access = listSlots->currentItem()->text( 1 );
+    slot.type = "slot";
     QValueList<MetaDataBase::Function>::Iterator it = w->lstSlots.find( slot );
     if ( it != w->lstSlots.end() )
 	w->lstSlots.remove( it );
