@@ -62,7 +62,7 @@ static int numberLength( const char *s )
 */
 static QCString zeroKey( const char *key )
 {
-    QCString zeroed( strlen(key) + 1 );
+    QCString zeroed( (int)strlen(key) + 1 );
     char *z = zeroed.data();
     int i = 0, j = 0;
     int len;
